@@ -1,5 +1,6 @@
 import { css } from '@emotion/css';
-import React, { MouseEvent as ReactMouseEvent, useCallback, useEffect, useRef, useState } from 'react';
+import { MouseEvent as ReactMouseEvent, useCallback, useEffect, useRef, useState } from 'react';
+import * as React from 'react';
 import { useMeasure } from 'react-use';
 
 import { PIXELS_PER_LEVEL } from '../constants';
@@ -7,7 +8,7 @@ import { ClickedItemData, ColorScheme, ColorSchemeDiff, SelectedView, TextAlign
 
 import FlameGraphContextMenu, { GetExtraContextMenuButtonsFunction } from './FlameGraphContextMenu';
 import FlameGraphTooltip from './FlameGraphTooltip';
-import { CollapseConfig, CollapsedMap, FlameGraphDataContainer, LevelItem } from './dataTransform';
+import { CollapsedMap, FlameGraphDataContainer, LevelItem } from './dataTransform';
 import { getBarX, useFlameRender } from './rendering';
 
 type Props = {
@@ -209,16 +210,16 @@ const FlameGraphCanvas = ({
             onSandwich(data.getLabel(clickedItemData.item.itemIndexes[0]));
           }}
           onExpandGroup={() => {
-            setCollapsedMap(setCollapsedStatus(collapsedMap, clickedItemData.item, false));
+            setCollapsedMap(collapsedMap.setCollapsedStatus(clickedItemData.item, false));
           }}
           onCollapseGroup={() => {
-            setCollapsedMap(setCollapsedStatus(collapsedMap, clickedItemData.item, true));
+            setCollapsedMap(collapsedMap.setCollapsedStatus(clickedItemData.item, true));
           }}
           onExpandAllGroups={() => {
-            setCollapsedMap(setAllCollapsedStatus(collapsedMap, false));
+            setCollapsedMap(collapsedMap.setAllCollapsedStatus(false));
           }}
           onCollapseAllGroups={() => {
-            setCollapsedMap(setAllCollapsedStatus(collapsedMap, true));
+            setCollapsedMap(collapsedMap.setAllCollapsedStatus(true));
           }}
           allGroupsCollapsed={Array.from(collapsedMap.values()).every((i) => i.collapsed)}
           allGroupsExpanded={Array.from(collapsedMap.values()).every((i) => !i.collapsed)}
@@ -231,27 +232,6 @@ const FlameGraphCanvas = ({
   );
 };
 
-function setCollapsedStatus(collapsedMap: CollapsedMap, item: LevelItem, collapsed: boolean) {
-  const newMap = new Map(collapsedMap);
-  const collapsedConfig = collapsedMap.get(item)!;
-  const newConfig = { ...collapsedConfig, collapsed };
-  for (const item of collapsedConfig.items) {
-    newMap.set(item, newConfig);
-  }
-  return newMap;
-}
-
-function setAllCollapsedStatus(collapsedMap: CollapsedMap, collapsed: boolean) {
-  const newMap = new Map(collapsedMap);
-  for (const item of collapsedMap.keys()) {
-    const collapsedConfig = collapsedMap.get(item)!;
-    const newConfig = { ...collapsedConfig, collapsed };
-    newMap.set(item, newConfig);
-  }
-
-  return newMap;
-}
-
 const getStyles = () => ({
   graph: css({
     label: 'graph',
@@ -291,7 +271,7 @@ export const convertPixelCoordinatesToBarCoordinates = (
   pixelsPerTick: number,
   totalTicks: number,
   rangeMin: number,
-  collapsedMap: Map<LevelItem, CollapseConfig>
+  collapsedMap: CollapsedMap
 ): LevelItem | undefined => {
   let next: LevelItem | undefined = root;
   let currentLevel = direction === 'children' ? 0 : depth - 1;
