@@ -1,5 +1,4 @@
 import { render, screen } from '@testing-library/react';
-import React from 'react';
 
 import { createTheme } from '@grafana/data';
 
@@ -42,7 +41,7 @@ describe('BigValue', () => {
         />
       );
 
-      expect(screen.getByText('50%')).toBeInTheDocument();
+      expect(screen.getByText('0.5%')).toBeInTheDocument();
     });
 
     it('should render without percent change', () => {
