@@ -1,9 +1,10 @@
 // Wrap the DocsContainer for storybook-dark-mode theme switching support.
 import { DocsContainer, DocsContextProps } from '@storybook/addon-docs';
-import React from 'react';
+import * as React from 'react';
 import { useDarkMode } from 'storybook-dark-mode';
 
 import { GrafanaLight, GrafanaDark } from '../../../.storybook/storybookTheme';
+import { GlobalStyles } from '../../themes';
 
 type Props = {
   context: DocsContextProps;
@@ -15,6 +16,7 @@ export const ThemedDocsContainer = ({ children, context }: Props) => {
 
   return (
     <DocsContainer theme={dark ? GrafanaDark : GrafanaLight} context={context}>
+      <GlobalStyles />
       {children}
     </DocsContainer>
   );
