@@ -8,8 +8,9 @@ import (
 	"github.com/urfave/cli/v2"
 
 	gcli "github.com/grafana/grafana/pkg/cmd/grafana-cli/commands"
-	gsrv "github.com/grafana/grafana/pkg/cmd/grafana-server/commands"
-	"github.com/grafana/grafana/pkg/cmd/grafana/apiserver"
+	"github.com/grafana/grafana/pkg/cmd/grafana-server/commands"
+	"github.com/grafana/grafana/pkg/server"
+	"github.com/grafana/grafana/pkg/services/apiserver/standalone"
 )
 
 // The following variables cannot be constants, since they can be overridden through the -X link flag
@@ -31,7 +32,7 @@ func main() {
 }
 
 func MainApp() *cli.App {
-	return &cli.App{
+	app := &cli.App{
 		Name:  "grafana",
 		Usage: "Grafana server and command line interface",
 		Authors: []*cli.Author{
@@ -43,30 +44,32 @@ func MainApp() *cli.App {
 		Version: version,
 		Commands: []*cli.Command{
 			gcli.CLICommand(version),
-			gsrv.ServerCommand(version, commit, enterpriseCommit, buildBranch, buildstamp),
-			{
-				// The kubernetes standalone apiserver service runner
-				Name:  "apiserver",
-				Usage: "run a standalone api service (experimental)",
-				// Skip parsing flags because the command line is actually managed by cobra
-				SkipFlagParsing: true,
-				Action: func(context *cli.Context) error {
-					// exit here because apiserver handles its own error output
-					os.Exit(apiserver.RunCLI(gsrv.ServerOptions{
-						Version:          version,
-						Commit:           commit,
-						EnterpriseCommit: enterpriseCommit,
-						BuildBranch:      buildBranch,
-						BuildStamp:       buildstamp,
-						Context:          context,
-					}))
-					return nil
-				},
-			},
+			commands.ServerCommand(version, commit, enterpriseCommit, buildBranch, buildstamp),
 		},
 		CommandNotFound:      cmdNotFound,
 		EnableBashCompletion: true,
 	}
+
+	// Set the global build info
+	buildInfo := standalone.BuildInfo{
+		Version:          version,
+		Commit:           commit,
+		EnterpriseCommit: enterpriseCommit,
+		BuildBranch:      buildBranch,
+		BuildStamp:       buildstamp,
+	}
+	commands.SetBuildInfo(buildInfo)
+
+	// Add the enterprise command line to build an api server
+	f, err := server.InitializeAPIServerFactory()
+	if err == nil {
+		cmd := f.GetCLICommand(buildInfo)
+		if cmd != nil {
+			app.Commands = append(app.Commands, cmd)
+		}
+	}
+
+	return app
 }
 
 func cmdNotFound(c *cli.Context, command string) {
