@@ -93,22 +93,19 @@ func (d dummyUsageStatProvider) GetUsageStats(ctx context.Context) map[string]an
 }
 
 func TestUsageStatsProviders(t *testing.T) {
-	provider1 := &dummyUsageStatProvider{stats: map[string]any{"my_stat_1": "val1", "my_stat_2": "val2"}}
-	provider2 := &dummyUsageStatProvider{stats: map[string]any{"my_stat_x": "valx", "my_stat_z": "valz"}}
+	provider := &dummyUsageStatProvider{stats: map[string]any{"my_stat_x": "valx", "my_stat_z": "valz"}}
 
 	store := dbtest.NewFakeDB()
 	statsService := statstest.NewFakeService()
 	mockSystemStats(statsService)
 	s := createService(t, setting.NewCfg(), store, statsService)
-	s.RegisterProviders([]registry.ProvidesUsageStats{provider1, provider2})
+	s.RegisterProviders([]registry.ProvidesUsageStats{provider})
 
-	m, err := s.collectAdditionalMetrics(context.Background())
+	report, err := s.usageStats.GetUsageReport(context.Background())
 	require.NoError(t, err, "Expected no error")
 
-	assert.Equal(t, "val1", m["my_stat_1"])
-	assert.Equal(t, "val2", m["my_stat_2"])
-	assert.Equal(t, "valx", m["my_stat_x"])
-	assert.Equal(t, "valz", m["my_stat_z"])
+	assert.Equal(t, "valx", report.Metrics["my_stat_x"])
+	assert.Equal(t, "valz", report.Metrics["my_stat_z"])
 }
 
 func TestFeatureUsageStats(t *testing.T) {
@@ -142,13 +139,13 @@ func TestCollectingUsageStats(t *testing.T) {
 	s := createService(t, &setting.Cfg{
 		ReportingEnabled:     true,
 		BuildVersion:         "5.0.0",
-		AnonymousEnabled:     true,
+		Anonymous:            setting.AnonymousSettings{Enabled: true},
 		BasicAuthEnabled:     true,
 		LDAPAuthEnabled:      true,
-		AuthProxyEnabled:     true,
+		AuthProxy:            setting.AuthProxySettings{Enabled: true},
 		Packaging:            "deb",
 		ReportingDistributor: "hosted-grafana",
-		RemoteCacheOptions: &setting.RemoteCacheOptions{
+		RemoteCacheOptions: &setting.RemoteCacheSettings{
 			Name: "database",
 		},
 	}, sqlStore, statsService,
