@@ -8,32 +8,33 @@ import (
 	"sync"
 
 	"github.com/grafana/grafana-plugin-sdk-go/backend"
+	"go.opentelemetry.io/otel/trace"
+	"go.opentelemetry.io/otel/trace/noop"
 
 	"github.com/grafana/grafana/pkg/plugins"
 	"github.com/grafana/grafana/pkg/plugins/auth"
 	"github.com/grafana/grafana/pkg/plugins/backendplugin"
 	"github.com/grafana/grafana/pkg/plugins/log"
-	"github.com/grafana/grafana/pkg/plugins/pfs"
 	"github.com/grafana/grafana/pkg/plugins/repo"
 	"github.com/grafana/grafana/pkg/plugins/storage"
 )
 
 type FakePluginInstaller struct {
-	AddFunc func(ctx context.Context, pluginID, version string, opts plugins.CompatOpts) error
+	AddFunc func(ctx context.Context, pluginID, version string, opts plugins.AddOpts) error
 	// Remove removes a plugin from the store.
-	RemoveFunc func(ctx context.Context, pluginID string) error
+	RemoveFunc func(ctx context.Context, pluginID, version string) error
 }
 
-func (i *FakePluginInstaller) Add(ctx context.Context, pluginID, version string, opts plugins.CompatOpts) error {
+func (i *FakePluginInstaller) Add(ctx context.Context, pluginID, version string, opts plugins.AddOpts) error {
 	if i.AddFunc != nil {
 		return i.AddFunc(ctx, pluginID, version, opts)
 	}
 	return nil
 }
 
-func (i *FakePluginInstaller) Remove(ctx context.Context, pluginID string) error {
+func (i *FakePluginInstaller) Remove(ctx context.Context, pluginID, version string) error {
 	if i.RemoveFunc != nil {
-		return i.RemoveFunc(ctx, pluginID)
+		return i.RemoveFunc(ctx, pluginID, version)
 	}
 	return nil
 }
@@ -70,6 +71,9 @@ type FakePluginClient struct {
 	backend.CheckHealthHandlerFunc
 	backend.QueryDataHandlerFunc
 	backend.CallResourceHandlerFunc
+	backend.MutateAdmissionFunc
+	backend.ValidateAdmissionFunc
+	backend.ConvertObjectsFunc
 	mutex sync.RWMutex
 
 	backendplugin.Plugin
@@ -122,6 +126,10 @@ func (pc *FakePluginClient) IsDecommissioned() bool {
 	return pc.decommissioned
 }
 
+func (pc *FakePluginClient) Target() backendplugin.Target {
+	return "test-target"
+}
+
 func (pc *FakePluginClient) CollectMetrics(ctx context.Context, req *backend.CollectMetricsRequest) (*backend.CollectMetricsResult, error) {
 	if pc.CollectMetricsHandlerFunc != nil {
 		return pc.CollectMetricsHandlerFunc(ctx, req)
@@ -166,6 +174,30 @@ func (pc *FakePluginClient) RunStream(_ context.Context, _ *backend.RunStreamReq
 	return plugins.ErrMethodNotImplemented
 }
 
+func (pc *FakePluginClient) ValidateAdmission(ctx context.Context, req *backend.AdmissionRequest) (*backend.ValidationResponse, error) {
+	if pc.ValidateAdmissionFunc != nil {
+		return pc.ValidateAdmissionFunc(ctx, req)
+	}
+
+	return nil, plugins.ErrMethodNotImplemented
+}
+
+func (pc *FakePluginClient) MutateAdmission(ctx context.Context, req *backend.AdmissionRequest) (*backend.MutationResponse, error) {
+	if pc.MutateAdmissionFunc != nil {
+		return pc.MutateAdmissionFunc(ctx, req)
+	}
+
+	return nil, plugins.ErrMethodNotImplemented
+}
+
+func (pc *FakePluginClient) ConvertObjects(ctx context.Context, req *backend.ConversionRequest) (*backend.ConversionResponse, error) {
+	if pc.ConvertObjectsFunc != nil {
+		return pc.ConvertObjectsFunc(ctx, req)
+	}
+
+	return nil, plugins.ErrMethodNotImplemented
+}
+
 type FakePluginRegistry struct {
 	Store map[string]*plugins.Plugin
 }
@@ -204,7 +236,7 @@ type FakePluginRepo struct {
 	GetPluginArchiveFunc      func(_ context.Context, pluginID, version string, _ repo.CompatOpts) (*repo.PluginArchive, error)
 	GetPluginArchiveByURLFunc func(_ context.Context, archiveURL string, _ repo.CompatOpts) (*repo.PluginArchive, error)
 	GetPluginArchiveInfoFunc  func(_ context.Context, pluginID, version string, _ repo.CompatOpts) (*repo.PluginArchiveInfo, error)
-	PluginVersionFunc         func(pluginID, version string, compatOpts repo.CompatOpts) (repo.VersionData, error)
+	PluginVersionFunc         func(_ context.Context, pluginID, version string, compatOpts repo.CompatOpts) (repo.VersionData, error)
 }
 
 // GetPluginArchive fetches the requested plugin archive.
@@ -233,13 +265,21 @@ func (r *FakePluginRepo) GetPluginArchiveInfo(ctx context.Context, pluginID, ver
 	return &repo.PluginArchiveInfo{}, nil
 }
 
-func (r *FakePluginRepo) PluginVersion(pluginID, version string, compatOpts repo.CompatOpts) (repo.VersionData, error) {
+func (r *FakePluginRepo) PluginVersion(ctx context.Context, pluginID, version string, compatOpts repo.CompatOpts) (repo.VersionData, error) {
 	if r.PluginVersionFunc != nil {
-		return r.PluginVersionFunc(pluginID, version, compatOpts)
+		return r.PluginVersionFunc(ctx, pluginID, version, compatOpts)
 	}
 	return repo.VersionData{}, nil
 }
 
+type fakeTracerProvider struct {
+	noop.TracerProvider
+}
+
+func InitializeNoopTracerForTest() trace.Tracer {
+	return fakeTracerProvider{}.Tracer("test")
+}
+
 type FakePluginStorage struct {
 	ExtractFunc func(_ context.Context, pluginID string, dirNameFunc storage.DirNameGeneratorFunc, z *zip.ReadCloser) (*storage.ExtractedPluginArchive, error)
 }
@@ -313,7 +353,7 @@ func NewFakeBackendProcessProvider() *FakeBackendProcessProvider {
 	}
 	f.BackendFactoryFunc = func(ctx context.Context, p *plugins.Plugin) backendplugin.PluginFactoryFunc {
 		f.Requested[p.ID]++
-		return func(pluginID string, _ log.Logger, _ func() []string) (backendplugin.Plugin, error) {
+		return func(pluginID string, _ log.Logger, _ trace.Tracer, _ func() []string) (backendplugin.Plugin, error) {
 			f.Invoked[pluginID]++
 			return &FakePluginClient{}, nil
 		}
@@ -364,35 +404,55 @@ func (f *FakeRoleRegistry) DeclarePluginRoles(_ context.Context, _ string, _ str
 	return f.ExpectedErr
 }
 
-type FakePluginFiles struct {
+type FakeActionSetRegistry struct {
+	ExpectedErr error
+}
+
+func NewFakeActionSetRegistry() *FakeActionSetRegistry {
+	return &FakeActionSetRegistry{}
+}
+
+func (f *FakeActionSetRegistry) RegisterActionSets(_ context.Context, _ string, _ []plugins.ActionSet) error {
+	return f.ExpectedErr
+}
+
+type FakePluginFS struct {
 	OpenFunc   func(name string) (fs.File, error)
 	RemoveFunc func() error
+	RelFunc    func(string) (string, error)
 
 	base string
 }
 
-func NewFakePluginFiles(base string) *FakePluginFiles {
-	return &FakePluginFiles{
+func NewFakePluginFS(base string) *FakePluginFS {
+	return &FakePluginFS{
 		base: base,
 	}
 }
 
-func (f *FakePluginFiles) Open(name string) (fs.File, error) {
+func (f *FakePluginFS) Open(name string) (fs.File, error) {
 	if f.OpenFunc != nil {
 		return f.OpenFunc(name)
 	}
 	return nil, nil
 }
 
-func (f *FakePluginFiles) Base() string {
+func (f *FakePluginFS) Rel(_ string) (string, error) {
+	if f.RelFunc != nil {
+		return f.RelFunc(f.base)
+	}
+	return "", nil
+}
+
+func (f *FakePluginFS) Base() string {
 	return f.base
 }
 
-func (f *FakePluginFiles) Files() ([]string, error) {
+func (f *FakePluginFS) Files() ([]string, error) {
 	return []string{}, nil
 }
 
-func (f *FakePluginFiles) Remove() error {
+func (f *FakePluginFS) Remove() error {
 	if f.RemoveFunc != nil {
 		return f.RemoveFunc()
 	}
@@ -456,7 +516,7 @@ func (f *FakeAuthService) HasExternalService(ctx context.Context, pluginID strin
 	return f.Result != nil, nil
 }
 
-func (f *FakeAuthService) RegisterExternalService(ctx context.Context, pluginID string, pType pfs.Type, svc *pfs.IAM) (*auth.ExternalService, error) {
+func (f *FakeAuthService) RegisterExternalService(ctx context.Context, pluginID string, pType string, svc *auth.IAM) (*auth.ExternalService, error) {
 	return f.Result, nil
 }
 
@@ -476,36 +536,36 @@ func (f *FakeDiscoverer) Discover(ctx context.Context, src plugins.PluginSource)
 }
 
 type FakeBootstrapper struct {
-	BootstrapFunc func(ctx context.Context, src plugins.PluginSource, bundles []*plugins.FoundBundle) ([]*plugins.Plugin, error)
+	BootstrapFunc func(ctx context.Context, src plugins.PluginSource, bundle *plugins.FoundBundle) ([]*plugins.Plugin, error)
 }
 
-func (f *FakeBootstrapper) Bootstrap(ctx context.Context, src plugins.PluginSource, bundles []*plugins.FoundBundle) ([]*plugins.Plugin, error) {
+func (f *FakeBootstrapper) Bootstrap(ctx context.Context, src plugins.PluginSource, bundle *plugins.FoundBundle) ([]*plugins.Plugin, error) {
 	if f.BootstrapFunc != nil {
-		return f.BootstrapFunc(ctx, src, bundles)
+		return f.BootstrapFunc(ctx, src, bundle)
 	}
 	return []*plugins.Plugin{}, nil
 }
 
 type FakeValidator struct {
-	ValidateFunc func(ctx context.Context, ps []*plugins.Plugin) ([]*plugins.Plugin, error)
+	ValidateFunc func(ctx context.Context, ps *plugins.Plugin) error
 }
 
-func (f *FakeValidator) Validate(ctx context.Context, ps []*plugins.Plugin) ([]*plugins.Plugin, error) {
+func (f *FakeValidator) Validate(ctx context.Context, ps *plugins.Plugin) error {
 	if f.ValidateFunc != nil {
 		return f.ValidateFunc(ctx, ps)
 	}
-	return []*plugins.Plugin{}, nil
+	return nil
 }
 
 type FakeInitializer struct {
-	IntializeFunc func(ctx context.Context, ps []*plugins.Plugin) ([]*plugins.Plugin, error)
+	IntializeFunc func(ctx context.Context, ps *plugins.Plugin) (*plugins.Plugin, error)
 }
 
-func (f *FakeInitializer) Initialize(ctx context.Context, ps []*plugins.Plugin) ([]*plugins.Plugin, error) {
+func (f *FakeInitializer) Initialize(ctx context.Context, ps *plugins.Plugin) (*plugins.Plugin, error) {
 	if f.IntializeFunc != nil {
 		return f.IntializeFunc(ctx, ps)
 	}
-	return []*plugins.Plugin{}, nil
+	return ps, nil
 }
 
 type FakeTerminator struct {
@@ -589,3 +649,7 @@ func (p *FakeBackendPlugin) Kill() {
 	defer p.mutex.Unlock()
 	p.Running = false
 }
+
+func (p *FakeBackendPlugin) Target() backendplugin.Target {
+	return "test-target"
+}
