@@ -13,7 +13,6 @@ import (
 	"github.com/grafana/grafana/pkg/plugins"
 	"github.com/grafana/grafana/pkg/plugins/config"
 	"github.com/grafana/grafana/pkg/plugins/log"
-	"github.com/grafana/grafana/pkg/services/featuremgmt"
 	"github.com/grafana/grafana/pkg/util"
 )
 
@@ -26,19 +25,17 @@ var (
 type Local struct {
 	log        log.Logger
 	production bool
-	features   featuremgmt.FeatureToggles
 }
 
-func NewLocalFinder(devMode bool, features featuremgmt.FeatureToggles) *Local {
+func NewLocalFinder(devMode bool) *Local {
 	return &Local{
 		production: !devMode,
 		log:        log.New("local.finder"),
-		features:   features,
 	}
 }
 
-func ProvideLocalFinder(cfg *config.Cfg) *Local {
-	return NewLocalFinder(cfg.DevMode, cfg.Features)
+func ProvideLocalFinder(cfg *config.PluginManagementCfg) *Local {
+	return NewLocalFinder(cfg.DevMode)
 }
 
 func (l *Local) Find(ctx context.Context, src plugins.PluginSource) ([]*plugins.FoundBundle, error) {
@@ -59,12 +56,7 @@ func (l *Local) Find(ctx context.Context, src plugins.PluginSource) ([]*plugins.
 			continue
 		}
 
-		followDistFolder := true
-		if src.PluginClass(ctx) == plugins.ClassCore &&
-			!l.features.IsEnabledGlobally(featuremgmt.FlagExternalCorePlugins) {
-			followDistFolder = false
-		}
-		paths, err := l.getAbsPluginJSONPaths(path, followDistFolder)
+		paths, err := l.getAbsPluginJSONPaths(path)
 		if err != nil {
 			return nil, err
 		}
@@ -167,7 +159,7 @@ func (l *Local) readPluginJSON(pluginJSONPath string) (plugins.JSONData, error)
 	return plugin, nil
 }
 
-func (l *Local) getAbsPluginJSONPaths(path string, followDistFolder bool) ([]string, error) {
+func (l *Local) getAbsPluginJSONPaths(path string) ([]string, error) {
 	var pluginJSONPaths []string
 
 	var err error
@@ -176,7 +168,7 @@ func (l *Local) getAbsPluginJSONPaths(path string, followDistFolder bool) ([]str
 		return []string{}, err
 	}
 
-	if err = walk(path, true, true, followDistFolder,
+	if err = walk(path, true, true,
 		func(currentPath string, fi os.FileInfo, err error) error {
 			if err != nil {
 				if errors.Is(err, os.ErrNotExist) {
