@@ -3,7 +3,6 @@ package repo
 import (
 	"context"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"net/http"
 	"net/url"
@@ -22,28 +21,30 @@ type Manager struct {
 }
 
 func ProvideService(cfg *config.PluginManagementCfg) (*Manager, error) {
-	baseURL, err := url.JoinPath(cfg.GrafanaComURL, "/api/plugins")
+	baseURL, err := url.JoinPath(cfg.GrafanaComAPIURL, "/plugins")
 	if err != nil {
 		return nil, err
 	}
 
 	return NewManager(ManagerCfg{
-		SkipTLSVerify: false,
-		BaseURL:       baseURL,
-		Logger:        log.NewPrettyLogger("plugin.repository"),
+		SkipTLSVerify:      false,
+		BaseURL:            baseURL,
+		Logger:             log.NewPrettyLogger("plugin.repository"),
+		GrafanaComAPIToken: cfg.GrafanaComAPIToken,
 	}), nil
 }
 
 type ManagerCfg struct {
-	SkipTLSVerify bool
-	BaseURL       string
-	Logger        log.PrettyLogger
+	SkipTLSVerify      bool
+	BaseURL            string
+	GrafanaComAPIToken string
+	Logger             log.PrettyLogger
 }
 
 func NewManager(cfg ManagerCfg) *Manager {
 	return &Manager{
 		baseURL: cfg.BaseURL,
-		client:  NewClient(cfg.SkipTLSVerify, cfg.Logger),
+		client:  NewClient(cfg.SkipTLSVerify, cfg.GrafanaComAPIToken, cfg.Logger),
 		log:     cfg.Logger,
 	}
 }
@@ -64,8 +65,8 @@ func (m *Manager) GetPluginArchiveByURL(ctx context.Context, pluginZipURL string
 }
 
 // GetPluginArchiveInfo returns the options for downloading the requested plugin (with optional `version`)
-func (m *Manager) GetPluginArchiveInfo(_ context.Context, pluginID, version string, compatOpts CompatOpts) (*PluginArchiveInfo, error) {
-	v, err := m.PluginVersion(pluginID, version, compatOpts)
+func (m *Manager) GetPluginArchiveInfo(ctx context.Context, pluginID, version string, compatOpts CompatOpts) (*PluginArchiveInfo, error) {
+	v, err := m.PluginVersion(ctx, pluginID, version, compatOpts)
 	if err != nil {
 		return nil, err
 	}
@@ -78,18 +79,13 @@ func (m *Manager) GetPluginArchiveInfo(_ context.Context, pluginID, version stri
 }
 
 // PluginVersion will return plugin version based on the requested information
-func (m *Manager) PluginVersion(pluginID, version string, compatOpts CompatOpts) (VersionData, error) {
-	versions, err := m.grafanaCompatiblePluginVersions(pluginID, compatOpts)
+func (m *Manager) PluginVersion(ctx context.Context, pluginID, version string, compatOpts CompatOpts) (VersionData, error) {
+	versions, err := m.grafanaCompatiblePluginVersions(ctx, pluginID, compatOpts)
 	if err != nil {
 		return VersionData{}, err
 	}
 
-	sysCompatOpts, exists := compatOpts.System()
-	if !exists {
-		return VersionData{}, errors.New("no system compatibility requirements set")
-	}
-
-	compatibleVer, err := SelectSystemCompatibleVersion(m.log, versions, pluginID, version, sysCompatOpts)
+	compatibleVer, err := SelectSystemCompatibleVersion(m.log, versions, pluginID, version, compatOpts)
 	if err != nil {
 		return VersionData{}, err
 	}
@@ -109,7 +105,7 @@ func (m *Manager) downloadURL(pluginID, version string) string {
 }
 
 // grafanaCompatiblePluginVersions will get version info from /api/plugins/$pluginID/versions
-func (m *Manager) grafanaCompatiblePluginVersions(pluginID string, compatOpts CompatOpts) ([]Version, error) {
+func (m *Manager) grafanaCompatiblePluginVersions(ctx context.Context, pluginID string, compatOpts CompatOpts) ([]Version, error) {
 	u, err := url.Parse(m.baseURL)
 	if err != nil {
 		return nil, err
@@ -117,7 +113,7 @@ func (m *Manager) grafanaCompatiblePluginVersions(pluginID string, compatOpts Co
 
 	u.Path = path.Join(u.Path, pluginID, "versions")
 
-	body, err := m.client.SendReq(u, compatOpts)
+	body, err := m.client.SendReq(ctx, u, compatOpts)
 	if err != nil {
 		return nil, err
 	}
