@@ -1,13 +1,18 @@
 package options
 
 import (
+	"log/slog"
 	"strconv"
 
-	"github.com/go-logr/logr"
 	"github.com/spf13/pflag"
+	genericfeatures "k8s.io/apiserver/pkg/features"
 	genericapiserver "k8s.io/apiserver/pkg/server"
+	utilfeature "k8s.io/apiserver/pkg/util/feature"
 	"k8s.io/component-base/logs"
 	"k8s.io/klog/v2"
+
+	"github.com/grafana/grafana/pkg/infra/log"
+	"github.com/grafana/grafana/pkg/infra/log/slogadapter"
 )
 
 type ExtraOptions struct {
@@ -36,10 +41,16 @@ func (o *ExtraOptions) Validate() []error {
 }
 
 func (o *ExtraOptions) ApplyTo(c *genericapiserver.RecommendedConfig) error {
-	logger := logr.New(newLogAdapter(o.Verbosity))
-	klog.SetLoggerWithOptions(logger, klog.ContextualLogger(true))
+	handler := slogadapter.New(log.New("grafana-apiserver"))
+	logger := slog.New(handler)
+	if err := utilfeature.DefaultMutableFeatureGate.SetFromMap(map[string]bool{
+		string(genericfeatures.APIServerTracing): false,
+	}); err != nil {
+		return err
+	}
+	klog.SetSlogLogger(logger)
 	if _, err := logs.GlogSetter(strconv.Itoa(o.Verbosity)); err != nil {
-		logger.Error(err, "failed to set log level")
+		logger.Error("failed to set log level", "error", err)
 	}
 	c.ExternalAddress = o.ExternalAddress
 	return nil
