@@ -48,18 +48,13 @@ func (s *CorrelationsService) createHandler(c *contextmodel.ReqContext) response
 		return response.Error(http.StatusBadRequest, "bad request data", err)
 	}
 	cmd.SourceUID = web.Params(c.Req)[":uid"]
-	cmd.OrgId = c.OrgID
+	cmd.OrgId = c.SignedInUser.GetOrgID()
 
 	correlation, err := s.CreateCorrelation(c.Req.Context(), cmd)
 	if err != nil {
 		if errors.Is(err, ErrSourceDataSourceDoesNotExists) || errors.Is(err, ErrTargetDataSourceDoesNotExists) {
 			return response.Error(http.StatusNotFound, "Data source not found", err)
 		}
-
-		if errors.Is(err, ErrSourceDataSourceReadOnly) {
-			return response.Error(http.StatusForbidden, "Data source is read only", err)
-		}
-
 		return response.Error(http.StatusInternalServerError, "Failed to add correlation", err)
 	}
 
@@ -96,7 +91,7 @@ func (s *CorrelationsService) deleteHandler(c *contextmodel.ReqContext) response
 	cmd := DeleteCorrelationCommand{
 		UID:       web.Params(c.Req)[":correlationUID"],
 		SourceUID: web.Params(c.Req)[":uid"],
-		OrgId:     c.OrgID,
+		OrgId:     c.SignedInUser.GetOrgID(),
 	}
 
 	err := s.DeleteCorrelation(c.Req.Context(), cmd)
@@ -109,8 +104,8 @@ func (s *CorrelationsService) deleteHandler(c *contextmodel.ReqContext) response
 			return response.Error(http.StatusNotFound, "Correlation not found", err)
 		}
 
-		if errors.Is(err, ErrSourceDataSourceReadOnly) {
-			return response.Error(http.StatusForbidden, "Data source is read only", err)
+		if errors.Is(err, ErrCorrelationReadOnly) {
+			return response.Error(http.StatusForbidden, "Correlation can only be edited via provisioning", err)
 		}
 
 		return response.Error(http.StatusInternalServerError, "Failed to delete correlation", err)
@@ -158,7 +153,7 @@ func (s *CorrelationsService) updateHandler(c *contextmodel.ReqContext) response
 
 	cmd.UID = web.Params(c.Req)[":correlationUID"]
 	cmd.SourceUID = web.Params(c.Req)[":uid"]
-	cmd.OrgId = c.OrgID
+	cmd.OrgId = c.SignedInUser.GetOrgID()
 
 	correlation, err := s.UpdateCorrelation(c.Req.Context(), cmd)
 	if err != nil {
@@ -170,8 +165,8 @@ func (s *CorrelationsService) updateHandler(c *contextmodel.ReqContext) response
 			return response.Error(http.StatusNotFound, "Correlation not found", err)
 		}
 
-		if errors.Is(err, ErrSourceDataSourceReadOnly) {
-			return response.Error(http.StatusForbidden, "Data source is read only", err)
+		if errors.Is(err, ErrCorrelationReadOnly) {
+			return response.Error(http.StatusForbidden, "Correlation can only be edited via provisioning", err)
 		}
 
 		return response.Error(http.StatusInternalServerError, "Failed to update correlation", err)
@@ -211,7 +206,7 @@ func (s *CorrelationsService) getCorrelationHandler(c *contextmodel.ReqContext)
 	query := GetCorrelationQuery{
 		UID:       web.Params(c.Req)[":correlationUID"],
 		SourceUID: web.Params(c.Req)[":uid"],
-		OrgId:     c.OrgID,
+		OrgId:     c.SignedInUser.GetOrgID(),
 	}
 
 	correlation, err := s.getCorrelation(c.Req.Context(), query)
@@ -257,7 +252,7 @@ type GetCorrelationResponse struct {
 func (s *CorrelationsService) getCorrelationsBySourceUIDHandler(c *contextmodel.ReqContext) response.Response {
 	query := GetCorrelationsBySourceUIDQuery{
 		SourceUID: web.Params(c.Req)[":uid"],
-		OrgId:     c.OrgID,
+		OrgId:     c.SignedInUser.GetOrgID(),
 	}
 
 	correlations, err := s.getCorrelationsBySourceUID(c.Req.Context(), query)
@@ -313,7 +308,7 @@ func (s *CorrelationsService) getCorrelationsHandler(c *contextmodel.ReqContext)
 	sourceUIDs := c.QueryStrings("sourceUID")
 
 	query := GetCorrelationsQuery{
-		OrgId:      c.OrgID,
+		OrgId:      c.SignedInUser.GetOrgID(),
 		Limit:      limit,
 		Page:       page,
 		SourceUIDs: sourceUIDs,
