@@ -11,26 +11,27 @@ import (
 )
 
 const (
+	DS_ACCESS_DIRECT  = "direct"
+	DS_ACCESS_PROXY   = "proxy"
+	DS_ALERTMANAGER   = "alertmanager"
+	DS_AZURE_MONITOR  = "grafana-azure-monitor-datasource"
+	DS_DYNATRACE      = "grafana-dynatrace-datasource"
+	DS_ES             = "elasticsearch"
+	DS_ES_OPEN_DISTRO = "grafana-es-open-distro-datasource"
+	DS_ES_OPENSEARCH  = "grafana-opensearch-datasource"
 	DS_GRAPHITE       = "graphite"
 	DS_INFLUXDB       = "influxdb"
 	DS_INFLUXDB_08    = "influxdb_08"
-	DS_ES             = "elasticsearch"
-	DS_PROMETHEUS     = "prometheus"
-	DS_ALERTMANAGER   = "alertmanager"
 	DS_JAEGER         = "jaeger"
 	DS_LOKI           = "loki"
-	DS_OPENTSDB       = "opentsdb"
-	DS_TEMPO          = "tempo"
-	DS_ZIPKIN         = "zipkin"
-	DS_MYSQL          = "mysql"
-	DS_POSTGRES       = "grafana-postgresql-datasource"
 	DS_MSSQL          = "mssql"
-	DS_ACCESS_DIRECT  = "direct"
-	DS_ACCESS_PROXY   = "proxy"
-	DS_ES_OPEN_DISTRO = "grafana-es-open-distro-datasource"
-	DS_ES_OPENSEARCH  = "grafana-opensearch-datasource"
-	DS_AZURE_MONITOR  = "grafana-azure-monitor-datasource"
+	DS_MYSQL          = "mysql"
+	DS_OPENTSDB       = "opentsdb"
+	DS_POSTGRES       = "grafana-postgresql-datasource"
+	DS_PROMETHEUS     = "prometheus"
+	DS_TEMPO          = "tempo"
 	DS_TESTDATA       = "grafana-testdata-datasource"
+	DS_ZIPKIN         = "zipkin"
 	// CustomHeaderName is the prefix that is used to store the name of a custom header.
 	CustomHeaderName = "httpHeaderName"
 	// CustomHeaderValue is the prefix that is used to store the value of a custom header.
@@ -62,6 +63,10 @@ type DataSource struct {
 	SecureJsonData    map[string][]byte `json:"secureJsonData"`
 	ReadOnly          bool              `json:"readOnly"`
 	UID               string            `json:"uid" xorm:"uid"`
+	// swagger:ignore
+	APIVersion string `json:"apiVersion" xorm:"api_version"`
+	// swagger:ignore
+	IsPrunable bool `xorm:"is_prunable"`
 
 	Created time.Time `json:"created,omitempty"`
 	Updated time.Time `json:"updated,omitempty"`
@@ -83,10 +88,6 @@ type TeamHTTPHeader struct {
 	Value  string `json:"value"`
 }
 
-func (ds DataSource) TeamHTTPHeaders() (*TeamHTTPHeaders, error) {
-	return GetTeamHTTPHeaders(ds.JsonData)
-}
-
 func GetTeamHTTPHeaders(jsonData *simplejson.Json) (*TeamHTTPHeaders, error) {
 	teamHTTPHeaders := &TeamHTTPHeaders{}
 	if jsonData == nil {
@@ -161,6 +162,10 @@ type AddDataSourceCommand struct {
 	JsonData        *simplejson.Json  `json:"jsonData"`
 	SecureJsonData  map[string]string `json:"secureJsonData"`
 	UID             string            `json:"uid"`
+	// swagger:ignore
+	APIVersion string `json:"apiVersion"`
+	// swagger:ignore
+	IsPrunable bool
 
 	OrgID                   int64             `json:"-"`
 	UserID                  int64             `json:"-"`
@@ -185,6 +190,10 @@ type UpdateDataSourceCommand struct {
 	SecureJsonData  map[string]string `json:"secureJsonData"`
 	Version         int               `json:"version"`
 	UID             string            `json:"uid"`
+	// swagger:ignore
+	APIVersion string `json:"apiVersion"`
+	// swagger:ignore
+	IsPrunable bool
 
 	OrgID                   int64             `json:"-"`
 	ID                      int64             `json:"-"`
@@ -192,6 +201,8 @@ type UpdateDataSourceCommand struct {
 	EncryptedSecureJsonData map[string][]byte `json:"-"`
 	UpdateSecretFn          UpdateSecretFn    `json:"-"`
 	IgnoreOldSecureJsonData bool              `json:"-"`
+
+	AllowLBACRuleUpdates bool `json:"-"`
 }
 
 // DeleteDataSourceCommand will delete a DataSource based on OrgID as well as the UID (preferred), ID, or Name.
@@ -234,11 +245,6 @@ type GetDataSourcesByTypeQuery struct {
 	AliasIDs []string
 }
 
-type GetDefaultDataSourceQuery struct {
-	OrgID int64
-	User  *user.SignedInUser
-}
-
 // GetDataSourceQuery will get a DataSource based on OrgID as well as the UID (preferred), ID, or Name.
 // At least one of the UID, ID, or Name properties must be set in addition to OrgID.
 type GetDataSourceQuery struct {
