@@ -7,10 +7,11 @@ type codeowner string
 const (
 	grafanaAppPlatformSquad                     codeowner = "@grafana/grafana-app-platform-squad"
 	grafanaDashboardsSquad                      codeowner = "@grafana/dashboards-squad"
-	grafanaExploreSquad                         codeowner = "@grafana/explore-squad"
 	grafanaDatavizSquad                         codeowner = "@grafana/dataviz-squad"
 	grafanaFrontendPlatformSquad                codeowner = "@grafana/grafana-frontend-platform"
-	grafanaBackendPlatformSquad                 codeowner = "@grafana/backend-platform"
+	grafanaBackendGroup                         codeowner = "@grafana/grafana-backend-group"
+	grafanaBackendServicesSquad                 codeowner = "@grafana/grafana-backend-services-squad"
+	grafanaSearchAndStorageSquad                codeowner = "@grafana/search-and-storage"
 	grafanaPluginsPlatformSquad                 codeowner = "@grafana/plugins-platform-backend"
 	grafanaAsCodeSquad                          codeowner = "@grafana/grafana-as-code"
 	identityAccessTeam                          codeowner = "@grafana/identity-access-team"
@@ -26,4 +27,6 @@ const (
 	enterpriseDatasourcesSquad                  codeowner = "@grafana/enterprise-datasources"
 	grafanaSharingSquad                         codeowner = "@grafana/sharing-squad"
 	grafanaDatabasesFrontend                    codeowner = "@grafana/databases-frontend"
+	grafanaOSSBigTent                           codeowner = "@grafana/oss-big-tent"
+	growthAndOnboarding                         codeowner = "@grafana/growth-and-onboarding"
 )
