@@ -34,7 +34,8 @@ func TestGetLibraryElement(t *testing.T) {
 					Result: libraryElement{
 						ID:          1,
 						OrgID:       1,
-						FolderID:    1,
+						FolderID:    1, // nolint:staticcheck
+						FolderUID:   sc.folder.UID,
 						UID:         res.Result.UID,
 						Name:        "Text - Library Panel",
 						Kind:        int64(model.PanelElement),
@@ -122,7 +123,8 @@ func TestGetLibraryElement(t *testing.T) {
 				Title: "Testing getHandler",
 				Data:  simplejson.NewFromAny(dashJSON),
 			}
-			dashInDB := createDashboard(t, sc.sqlStore, sc.user, &dash, sc.folder.ID)
+			// nolint:staticcheck
+			dashInDB := createDashboard(t, sc.sqlStore, sc.user, &dash, sc.folder.ID, sc.folder.UID)
 			err := sc.service.ConnectElementsToDashboard(sc.reqContext.Req.Context(), sc.reqContext.SignedInUser, []string{sc.initialResult.Result.UID}, dashInDB.ID)
 			require.NoError(t, err)
 
@@ -131,7 +133,8 @@ func TestGetLibraryElement(t *testing.T) {
 					Result: libraryElement{
 						ID:          1,
 						OrgID:       1,
-						FolderID:    1,
+						FolderID:    1, // nolint:staticcheck
+						FolderUID:   sc.folder.UID,
 						UID:         res.Result.UID,
 						Name:        "Text - Library Panel",
 						Kind:        int64(model.PanelElement),
