@@ -68,12 +68,18 @@ func (r *LotexRuler) RouteDeleteNamespaceRulesConfig(ctx *contextmodel.ReqContex
 	if err != nil {
 		return ErrResp(500, err, "")
 	}
+
+	finalNamespace, err := getRulesNamespaceParam(ctx, namespace)
+	if err != nil {
+		return ErrResp(http.StatusBadRequest, err, "")
+	}
+
 	return r.requester.withReq(
 		ctx,
 		http.MethodDelete,
 		withPath(
 			*ctx.Req.URL,
-			fmt.Sprintf("%s/%s", legacyRulerPrefix, url.PathEscape(namespace)),
+			fmt.Sprintf("%s/%s", legacyRulerPrefix, url.PathEscape(finalNamespace)),
 		),
 		nil,
 		messageExtractor,
@@ -86,6 +92,17 @@ func (r *LotexRuler) RouteDeleteRuleGroupConfig(ctx *contextmodel.ReqContext, na
 	if err != nil {
 		return ErrResp(500, err, "")
 	}
+
+	finalNamespace, err := getRulesNamespaceParam(ctx, namespace)
+	if err != nil {
+		return ErrResp(http.StatusBadRequest, err, "")
+	}
+
+	finalGroup, err := getRulesGroupParam(ctx, group)
+	if err != nil {
+		return ErrResp(http.StatusBadRequest, err, "")
+	}
+
 	return r.requester.withReq(
 		ctx,
 		http.MethodDelete,
@@ -94,8 +111,8 @@ func (r *LotexRuler) RouteDeleteRuleGroupConfig(ctx *contextmodel.ReqContext, na
 			fmt.Sprintf(
 				"%s/%s/%s",
 				legacyRulerPrefix,
-				url.PathEscape(namespace),
-				url.PathEscape(group),
+				url.PathEscape(finalNamespace),
+				url.PathEscape(finalGroup),
 			),
 		),
 		nil,
@@ -109,6 +126,12 @@ func (r *LotexRuler) RouteGetNamespaceRulesConfig(ctx *contextmodel.ReqContext,
 	if err != nil {
 		return ErrResp(500, err, "")
 	}
+
+	finalNamespace, err := getRulesNamespaceParam(ctx, namespace)
+	if err != nil {
+		return ErrResp(http.StatusBadRequest, err, "")
+	}
+
 	return r.requester.withReq(
 		ctx,
 		http.MethodGet,
@@ -117,7 +140,7 @@ func (r *LotexRuler) RouteGetNamespaceRulesConfig(ctx *contextmodel.ReqContext,
 			fmt.Sprintf(
 				"%s/%s",
 				legacyRulerPrefix,
-				url.PathEscape(namespace),
+				url.PathEscape(finalNamespace),
 			),
 		),
 		nil,
@@ -131,6 +154,17 @@ func (r *LotexRuler) RouteGetRulegGroupConfig(ctx *contextmodel.ReqContext, name
 	if err != nil {
 		return ErrResp(500, err, "")
 	}
+
+	finalNamespace, err := getRulesNamespaceParam(ctx, namespace)
+	if err != nil {
+		return ErrResp(http.StatusBadRequest, err, "")
+	}
+
+	finalGroup, err := getRulesGroupParam(ctx, group)
+	if err != nil {
+		return ErrResp(http.StatusBadRequest, err, "")
+	}
+
 	return r.requester.withReq(
 		ctx,
 		http.MethodGet,
@@ -139,8 +173,8 @@ func (r *LotexRuler) RouteGetRulegGroupConfig(ctx *contextmodel.ReqContext, name
 			fmt.Sprintf(
 				"%s/%s/%s",
 				legacyRulerPrefix,
-				url.PathEscape(namespace),
-				url.PathEscape(group),
+				url.PathEscape(finalNamespace),
+				url.PathEscape(finalGroup),
 			),
 		),
 		nil,
@@ -177,7 +211,13 @@ func (r *LotexRuler) RoutePostNameRulesConfig(ctx *contextmodel.ReqContext, conf
 	if err != nil {
 		return ErrResp(500, err, "Failed marshal rule group")
 	}
-	u := withPath(*ctx.Req.URL, fmt.Sprintf("%s/%s", legacyRulerPrefix, ns))
+
+	finalNamespace, err := getRulesNamespaceParam(ctx, ns)
+	if err != nil {
+		return ErrResp(http.StatusBadRequest, err, "")
+	}
+
+	u := withPath(*ctx.Req.URL, fmt.Sprintf("%s/%s", legacyRulerPrefix, url.PathEscape(finalNamespace)))
 	return r.requester.withReq(ctx, http.MethodPost, u, bytes.NewBuffer(yml), jsonExtractor(nil), nil)
 }
 
