@@ -74,6 +74,11 @@ func TestValidate(t *testing.T) {
 			notificationSettings: CopyNotificationSettings(validNotificationSettings(), NSMuts.WithGroupInterval(util.Pointer(-1*time.Second))),
 			expErrorContains:     "group interval",
 		},
+		{
+			name:                 "group interval zero is invalid",
+			notificationSettings: CopyNotificationSettings(validNotificationSettings(), NSMuts.WithGroupInterval(util.Pointer(0*time.Second))),
+			expErrorContains:     "group interval",
+		},
 		{
 			name:                 "repeat interval empty is valid",
 			notificationSettings: CopyNotificationSettings(validNotificationSettings(), NSMuts.WithRepeatInterval(nil)),
@@ -87,6 +92,11 @@ func TestValidate(t *testing.T) {
 			notificationSettings: CopyNotificationSettings(validNotificationSettings(), NSMuts.WithRepeatInterval(util.Pointer(-1*time.Second))),
 			expErrorContains:     "repeat interval",
 		},
+		{
+			name:                 "repeat interval zero is invalid",
+			notificationSettings: CopyNotificationSettings(validNotificationSettings(), NSMuts.WithRepeatInterval(util.Pointer(0*time.Second))),
+			expErrorContains:     "repeat interval",
+		},
 	}
 
 	for _, tt := range testCases {
