@@ -4,38 +4,56 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
+	"slices"
 	"strings"
 	"testing"
 
+	"github.com/grafana/alerting/notify"
 	"github.com/prometheus/alertmanager/config"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+	"golang.org/x/exp/maps"
 
 	"github.com/grafana/grafana/pkg/components/simplejson"
 	"github.com/grafana/grafana/pkg/infra/db"
 	"github.com/grafana/grafana/pkg/infra/log"
+	"github.com/grafana/grafana/pkg/infra/tracing"
 	"github.com/grafana/grafana/pkg/services/accesscontrol"
 	"github.com/grafana/grafana/pkg/services/accesscontrol/acimpl"
-	"github.com/grafana/grafana/pkg/services/accesscontrol/actest"
+	"github.com/grafana/grafana/pkg/services/featuremgmt"
+	ac "github.com/grafana/grafana/pkg/services/ngalert/accesscontrol"
 	"github.com/grafana/grafana/pkg/services/ngalert/api/tooling/definitions"
 	"github.com/grafana/grafana/pkg/services/ngalert/models"
 	"github.com/grafana/grafana/pkg/services/ngalert/notifier"
+	"github.com/grafana/grafana/pkg/services/ngalert/notifier/channels_config"
+	"github.com/grafana/grafana/pkg/services/ngalert/notifier/legacy_storage"
 	"github.com/grafana/grafana/pkg/services/ngalert/tests/fakes"
 	"github.com/grafana/grafana/pkg/services/secrets"
 	"github.com/grafana/grafana/pkg/services/secrets/database"
 	"github.com/grafana/grafana/pkg/services/secrets/manager"
 	"github.com/grafana/grafana/pkg/services/user"
-	"github.com/grafana/grafana/pkg/setting"
 	"github.com/grafana/grafana/pkg/util"
 )
 
 func TestContactPointService(t *testing.T) {
 	sqlStore := db.InitTestDB(t)
 	secretsService := manager.SetupTestService(t, database.ProvideSecretsStore(sqlStore))
+
+	redactedUser := &user.SignedInUser{OrgID: 1, Permissions: map[int64]map[string][]string{
+		1: {
+			accesscontrol.ActionAlertingProvisioningRead: nil,
+		},
+	}}
+	decryptedUser := &user.SignedInUser{OrgID: 1, Permissions: map[int64]map[string][]string{
+		1: {
+			accesscontrol.ActionAlertingProvisioningReadSecrets: nil,
+		},
+	}}
+
 	t.Run("service gets contact points from AM config", func(t *testing.T) {
 		sut := createContactPointServiceSut(t, secretsService)
 
-		cps, err := sut.GetContactPoints(context.Background(), cpsQuery(1), nil)
+		cps, err := sut.GetContactPoints(context.Background(), cpsQuery(1), redactedUser)
 		require.NoError(t, err)
 
 		require.Len(t, cps, 2)
@@ -46,7 +64,7 @@ func TestContactPointService(t *testing.T) {
 	t.Run("service filters contact points by name", func(t *testing.T) {
 		sut := createContactPointServiceSut(t, secretsService)
 
-		cps, err := sut.GetContactPoints(context.Background(), cpsQueryWithName(1, "slack receiver"), nil)
+		cps, err := sut.GetContactPoints(context.Background(), cpsQueryWithName(1, "slack receiver"), redactedUser)
 		require.NoError(t, err)
 
 		require.Len(t, cps, 1)
@@ -56,7 +74,7 @@ func TestContactPointService(t *testing.T) {
 	t.Run("service filters contact points by name, returns empty when no match", func(t *testing.T) {
 		sut := createContactPointServiceSut(t, secretsService)
 
-		cps, err := sut.GetContactPoints(context.Background(), cpsQueryWithName(1, "unknown"), nil)
+		cps, err := sut.GetContactPoints(context.Background(), cpsQueryWithName(1, "unknown"), redactedUser)
 		require.NoError(t, err)
 
 		require.Len(t, cps, 0)
@@ -66,10 +84,10 @@ func TestContactPointService(t *testing.T) {
 		sut := createContactPointServiceSut(t, secretsService)
 		newCp := createTestContactPoint()
 
-		_, err := sut.CreateContactPoint(context.Background(), 1, newCp, models.ProvenanceAPI)
+		_, err := sut.CreateContactPoint(context.Background(), 1, redactedUser, newCp, models.ProvenanceAPI)
 		require.NoError(t, err)
 
-		cps, err := sut.GetContactPoints(context.Background(), cpsQuery(1), nil)
+		cps, err := sut.GetContactPoints(context.Background(), cpsQuery(1), redactedUser)
 		require.NoError(t, err)
 		require.Len(t, cps, 3)
 		require.Equal(t, "test-contact-point", cps[2].Name)
@@ -82,10 +100,10 @@ func TestContactPointService(t *testing.T) {
 		newCp := createTestContactPoint()
 		newCp.UID = customUID
 
-		_, err := sut.CreateContactPoint(context.Background(), 1, newCp, models.ProvenanceAPI)
+		_, err := sut.CreateContactPoint(context.Background(), 1, redactedUser, newCp, models.ProvenanceAPI)
 		require.NoError(t, err)
 
-		cps, err := sut.GetContactPoints(context.Background(), cpsQueryWithName(1, newCp.Name), nil)
+		cps, err := sut.GetContactPoints(context.Background(), cpsQueryWithName(1, newCp.Name), redactedUser)
 		require.NoError(t, err)
 		require.Len(t, cps, 1)
 		require.Equal(t, customUID, cps[0].UID)
@@ -97,7 +115,7 @@ func TestContactPointService(t *testing.T) {
 		newCp := createTestContactPoint()
 		newCp.UID = customUID
 
-		_, err := sut.CreateContactPoint(context.Background(), 1, newCp, models.ProvenanceAPI)
+		_, err := sut.CreateContactPoint(context.Background(), 1, redactedUser, newCp, models.ProvenanceAPI)
 		require.ErrorIs(t, err, ErrValidation)
 	})
 
@@ -107,10 +125,10 @@ func TestContactPointService(t *testing.T) {
 		newCp := createTestContactPoint()
 		newCp.UID = customUID
 
-		_, err := sut.CreateContactPoint(context.Background(), 1, newCp, models.ProvenanceAPI)
+		_, err := sut.CreateContactPoint(context.Background(), 1, redactedUser, newCp, models.ProvenanceAPI)
 		require.NoError(t, err)
 
-		_, err = sut.CreateContactPoint(context.Background(), 1, newCp, models.ProvenanceAPI)
+		_, err = sut.CreateContactPoint(context.Background(), 1, redactedUser, newCp, models.ProvenanceAPI)
 		require.Error(t, err)
 	})
 
@@ -119,7 +137,7 @@ func TestContactPointService(t *testing.T) {
 		newCp := createTestContactPoint()
 		newCp.Type = ""
 
-		_, err := sut.CreateContactPoint(context.Background(), 1, newCp, models.ProvenanceAPI)
+		_, err := sut.CreateContactPoint(context.Background(), 1, redactedUser, newCp, models.ProvenanceAPI)
 
 		require.ErrorIs(t, err, ErrValidation)
 	})
@@ -127,7 +145,7 @@ func TestContactPointService(t *testing.T) {
 	t.Run("update rejects contact points with no settings", func(t *testing.T) {
 		sut := createContactPointServiceSut(t, secretsService)
 		newCp := createTestContactPoint()
-		newCp, err := sut.CreateContactPoint(context.Background(), 1, newCp, models.ProvenanceAPI)
+		newCp, err := sut.CreateContactPoint(context.Background(), 1, redactedUser, newCp, models.ProvenanceAPI)
 		require.NoError(t, err)
 		newCp.Settings = nil
 
@@ -139,7 +157,7 @@ func TestContactPointService(t *testing.T) {
 	t.Run("update rejects contact points with no type", func(t *testing.T) {
 		sut := createContactPointServiceSut(t, secretsService)
 		newCp := createTestContactPoint()
-		newCp, err := sut.CreateContactPoint(context.Background(), 1, newCp, models.ProvenanceAPI)
+		newCp, err := sut.CreateContactPoint(context.Background(), 1, redactedUser, newCp, models.ProvenanceAPI)
 		require.NoError(t, err)
 		newCp.Type = ""
 
@@ -151,7 +169,7 @@ func TestContactPointService(t *testing.T) {
 	t.Run("update rejects contact points which fail validation after merging", func(t *testing.T) {
 		sut := createContactPointServiceSut(t, secretsService)
 		newCp := createTestContactPoint()
-		newCp, err := sut.CreateContactPoint(context.Background(), 1, newCp, models.ProvenanceAPI)
+		newCp, err := sut.CreateContactPoint(context.Background(), 1, redactedUser, newCp, models.ProvenanceAPI)
 		require.NoError(t, err)
 		newCp.Settings, _ = simplejson.NewJson([]byte(`{}`))
 
@@ -160,10 +178,48 @@ func TestContactPointService(t *testing.T) {
 		require.ErrorIs(t, err, ErrValidation)
 	})
 
+	t.Run("update renames references when group is renamed", func(t *testing.T) {
+		cfg := createEncryptedConfig(t, secretsService)
+		store := fakes.NewFakeAlertmanagerConfigStore(cfg)
+		sut := createContactPointServiceSutWithConfigStore(t, secretsService, store)
+
+		svc := &fakeReceiverService{}
+		sut.receiverService = svc
+
+		newCp := createTestContactPoint()
+		oldName := newCp.Name
+		newName := "new-name"
+
+		newCp, err := sut.CreateContactPoint(context.Background(), 1, redactedUser, newCp, models.ProvenanceAPI)
+		require.NoError(t, err)
+
+		newCp.Name = newName
+
+		svc.RenameReceiverInDependentResourcesFunc = func(ctx context.Context, orgID int64, route *definitions.Route, oldName, newName string, receiverProvenance models.Provenance) error {
+			legacy_storage.RenameReceiverInRoute(oldName, newName, route)
+			return nil
+		}
+
+		err = sut.UpdateContactPoint(context.Background(), 1, newCp, models.ProvenanceAPI)
+		require.NoError(t, err)
+
+		parsed, err := legacy_storage.DeserializeAlertmanagerConfig([]byte(store.LastSaveCommand.AlertmanagerConfiguration))
+		require.NoError(t, err)
+
+		require.Lenf(t, svc.Calls, 1, "service was supposed to be called once")
+		assert.Equal(t, "RenameReceiverInDependentResources", svc.Calls[0].Method)
+		assertInTransaction(t, svc.Calls[0].Args[0].(context.Context))
+		assert.Equal(t, int64(1), svc.Calls[0].Args[1])
+		assert.EqualValues(t, parsed.AlertmanagerConfig.Route, svc.Calls[0].Args[2])
+		assert.Equal(t, oldName, svc.Calls[0].Args[3])
+		assert.Equal(t, newName, svc.Calls[0].Args[4])
+		assert.Equal(t, models.ProvenanceAPI, svc.Calls[0].Args[5])
+	})
+
 	t.Run("default provenance of contact points is none", func(t *testing.T) {
 		sut := createContactPointServiceSut(t, secretsService)
 
-		cps, err := sut.GetContactPoints(context.Background(), cpsQuery(1), nil)
+		cps, err := sut.GetContactPoints(context.Background(), cpsQuery(1), redactedUser)
 		require.NoError(t, err)
 
 		require.Equal(t, models.ProvenanceNone, models.Provenance(cps[0].Provenance))
@@ -218,10 +274,10 @@ func TestContactPointService(t *testing.T) {
 				sut := createContactPointServiceSut(t, secretsService)
 				newCp := createTestContactPoint()
 
-				newCp, err := sut.CreateContactPoint(context.Background(), 1, newCp, test.from)
+				newCp, err := sut.CreateContactPoint(context.Background(), 1, redactedUser, newCp, test.from)
 				require.NoError(t, err)
 
-				cps, err := sut.GetContactPoints(context.Background(), cpsQueryWithName(1, newCp.Name), nil)
+				cps, err := sut.GetContactPoints(context.Background(), cpsQueryWithName(1, newCp.Name), redactedUser)
 				require.NoError(t, err)
 				require.Equal(t, newCp.UID, cps[0].UID)
 				require.Equal(t, test.from, models.Provenance(cps[0].Provenance))
@@ -230,7 +286,7 @@ func TestContactPointService(t *testing.T) {
 				if test.errNil {
 					require.NoError(t, err)
 
-					cps, err = sut.GetContactPoints(context.Background(), cpsQueryWithName(1, newCp.Name), nil)
+					cps, err = sut.GetContactPoints(context.Background(), cpsQueryWithName(1, newCp.Name), redactedUser)
 					require.NoError(t, err)
 					require.Equal(t, newCp.UID, cps[0].UID)
 					require.Equal(t, test.to, models.Provenance(cps[0].Provenance))
@@ -242,39 +298,87 @@ func TestContactPointService(t *testing.T) {
 	})
 
 	t.Run("service respects concurrency token when updating", func(t *testing.T) {
-		sut := createContactPointServiceSut(t, secretsService)
+		cfg := createEncryptedConfig(t, secretsService)
+		fakeConfigStore := fakes.NewFakeAlertmanagerConfigStore(cfg)
+		sut := createContactPointServiceSutWithConfigStore(t, secretsService, fakeConfigStore)
 		newCp := createTestContactPoint()
-		config, err := sut.configStore.store.GetLatestAlertmanagerConfiguration(context.Background(), 1)
+		config, err := sut.configStore.Get(context.Background(), 1)
 		require.NoError(t, err)
-		expectedConcurrencyToken := config.ConfigurationHash
+		expectedConcurrencyToken := config.ConcurrencyToken
 
-		_, err = sut.CreateContactPoint(context.Background(), 1, newCp, models.ProvenanceAPI)
+		_, err = sut.CreateContactPoint(context.Background(), 1, redactedUser, newCp, models.ProvenanceAPI)
 		require.NoError(t, err)
 
-		fake := sut.configStore.store.(*fakes.FakeAlertmanagerConfigStore)
-		intercepted := fake.LastSaveCommand
+		intercepted := fakeConfigStore.LastSaveCommand
 		require.Equal(t, expectedConcurrencyToken, intercepted.FetchedConfigurationHash)
 	})
+
+	t.Run("secrets are parsed in a case-insensitive way", func(t *testing.T) {
+		// JSON unmarshalling is case-insensitive. This means we can have
+		// a setting named "TOKEN" instead of "token". This test ensures that
+		// we handle such cases correctly and the token value is properly parsed,
+		// even if the setting key does not match the JSON key exactly.
+		tests := []struct {
+			settingsJSON  string
+			expectedValue string
+			name          string
+		}{
+			{
+				settingsJSON:  `{"recipient":"value_recipient","TOKEN":"some-other-token"}`,
+				expectedValue: "some-other-token",
+				name:          "token key is uppercased",
+			},
+
+			// This test checks that if multiple token keys are present in the settings,
+			// the key with the exact matching name is used.
+			{
+				settingsJSON:  `{"recipient":"value_recipient","TOKEN":"some-other-token", "token": "second-token"}`,
+				expectedValue: "second-token",
+				name:          "multiple token keys",
+			},
+		}
+
+		for _, tc := range tests {
+			t.Run(tc.name, func(t *testing.T) {
+				sut := createContactPointServiceSut(t, secretsService)
+
+				newCp := createTestContactPoint()
+				settings, _ := simplejson.NewJson([]byte(tc.settingsJSON))
+				newCp.Settings = settings
+
+				_, err := sut.CreateContactPoint(context.Background(), 1, redactedUser, newCp, models.ProvenanceAPI)
+				require.NoError(t, err)
+
+				q := cpsQueryWithName(1, newCp.Name)
+				q.Decrypt = true
+				cps, err := sut.GetContactPoints(context.Background(), q, decryptedUser)
+				require.NoError(t, err)
+				require.Len(t, cps, 1)
+				require.Equal(t, tc.expectedValue, cps[0].Settings.Get("token").MustString())
+			})
+		}
+	})
 }
 
 func TestContactPointServiceDecryptRedact(t *testing.T) {
 	secretsService := manager.SetupTestService(t, database.ProvideSecretsStore(db.InitTestDB(t)))
-	receiverServiceWithAC := func(ecp *ContactPointService) *notifier.ReceiverService {
-		return notifier.NewReceiverService(
-			acimpl.ProvideAccessControl(setting.NewCfg()),
-			// Get won't use the sut's config store, so we can use a different one here.
-			fakes.NewFakeAlertmanagerConfigStore(createEncryptedConfig(t, secretsService)),
-			ecp.provenanceStore,
-			ecp.encryptionService,
-			ecp.xact,
-			log.NewNopLogger(),
-		)
-	}
+
+	redactedUser := &user.SignedInUser{OrgID: 1, Permissions: map[int64]map[string][]string{
+		1: {
+			accesscontrol.ActionAlertingProvisioningRead: nil,
+		},
+	}}
+
+	decryptedUser := &user.SignedInUser{OrgID: 1, Permissions: map[int64]map[string][]string{
+		1: {
+			accesscontrol.ActionAlertingProvisioningReadSecrets: nil,
+		},
+	}}
 
 	t.Run("GetContactPoints gets redacted contact points by default", func(t *testing.T) {
 		sut := createContactPointServiceSut(t, secretsService)
 
-		cps, err := sut.GetContactPoints(context.Background(), cpsQuery(1), nil)
+		cps, err := sut.GetContactPoints(context.Background(), cpsQuery(1), redactedUser)
 		require.NoError(t, err)
 
 		require.Len(t, cps, 2)
@@ -284,35 +388,28 @@ func TestContactPointServiceDecryptRedact(t *testing.T) {
 
 	t.Run("GetContactPoints errors when Decrypt = true and user does not have permissions", func(t *testing.T) {
 		sut := createContactPointServiceSut(t, secretsService)
-		sut.receiverService = receiverServiceWithAC(sut)
 
 		q := cpsQuery(1)
 		q.Decrypt = true
-		_, err := sut.GetContactPoints(context.Background(), q, nil)
-		require.ErrorIs(t, err, ErrPermissionDenied)
+		_, err := sut.GetContactPoints(context.Background(), q, redactedUser)
+		require.ErrorIs(t, err, ac.ErrAuthorizationBase)
 	})
 	t.Run("GetContactPoints errors when Decrypt = true and user is nil", func(t *testing.T) {
 		sut := createContactPointServiceSut(t, secretsService)
-		sut.receiverService = receiverServiceWithAC(sut)
 
 		q := cpsQuery(1)
 		q.Decrypt = true
 		_, err := sut.GetContactPoints(context.Background(), q, nil)
-		require.ErrorIs(t, err, ErrPermissionDenied)
+		require.ErrorIs(t, err, ac.ErrAuthorizationBase)
 	})
 
 	t.Run("GetContactPoints gets decrypted contact points when Decrypt = true and user has permissions", func(t *testing.T) {
 		sut := createContactPointServiceSut(t, secretsService)
-		sut.receiverService = receiverServiceWithAC(sut)
 
 		expectedName := "slack receiver"
 		q := cpsQueryWithName(1, expectedName)
 		q.Decrypt = true
-		cps, err := sut.GetContactPoints(context.Background(), q, &user.SignedInUser{OrgID: 1, Permissions: map[int64]map[string][]string{
-			1: {
-				accesscontrol.ActionAlertingProvisioningReadSecrets: nil,
-			},
-		}})
+		cps, err := sut.GetContactPoints(context.Background(), q, decryptedUser)
 		require.NoError(t, err)
 
 		require.Len(t, cps, 1)
@@ -321,61 +418,95 @@ func TestContactPointServiceDecryptRedact(t *testing.T) {
 	})
 }
 
-func TestContactPointInUse(t *testing.T) {
-	result := isContactPointInUse("test", []*definitions.Route{
-		{
-			Receiver: "not-test",
-			Routes: []*definitions.Route{
-				{
-					Receiver: "not-test",
-				},
-				{
-					Receiver: "test",
-				},
-			},
+func TestRemoveSecretsForContactPoint(t *testing.T) {
+	overrides := map[string]func(settings map[string]any){
+		"webhook": func(settings map[string]any) { // add additional field to the settings because valid config does not allow it to be specified along with password
+			settings["authorization_credentials"] = "test-authz-creds"
 		},
-	})
-	require.True(t, result)
-	result = isContactPointInUse("test", []*definitions.Route{
-		{
-			Receiver: "not-test",
-			Routes: []*definitions.Route{
-				{
-					Receiver: "not-test",
-				},
-				{
-					Receiver: "not-test",
-				},
-			},
-		},
-	})
-	require.False(t, result)
+	}
+
+	configs := notify.AllKnownConfigsForTesting
+	keys := maps.Keys(configs)
+	slices.Sort(keys)
+	for _, integrationType := range keys {
+		cfg := configs[integrationType]
+		var settings map[string]any
+		require.NoError(t, json.Unmarshal([]byte(cfg.Config), &settings))
+		if f, ok := overrides[integrationType]; ok {
+			f(settings)
+		}
+		settingsRaw, err := json.Marshal(settings)
+		require.NoError(t, err)
+
+		expectedFields, err := channels_config.GetSecretKeysForContactPointType(integrationType)
+		require.NoError(t, err)
+
+		t.Run(integrationType, func(t *testing.T) {
+			cp := definitions.EmbeddedContactPoint{
+				Name:     "integration-" + integrationType,
+				Type:     integrationType,
+				Settings: simplejson.MustJson(settingsRaw),
+			}
+			secureFields, err := RemoveSecretsForContactPoint(&cp)
+			require.NoError(t, err)
+
+		FIELDS_ASSERT:
+			for _, field := range expectedFields {
+				assert.Contains(t, secureFields, field)
+				path := strings.Split(field, ".")
+				var expectedValue any = settings
+				for _, segment := range path {
+					v, ok := expectedValue.(map[string]any)
+					if !ok {
+						assert.Fail(t, fmt.Sprintf("cannot get expected value for field '%s'", field))
+						continue FIELDS_ASSERT
+					}
+					expectedValue = v[segment]
+				}
+				assert.EqualValues(t, secureFields[field], expectedValue)
+				v, err := cp.Settings.GetPath(path...).Value()
+				assert.NoError(t, err)
+				assert.Nilf(t, v, "field %s is expected to be removed from the settings", field)
+			}
+		})
+	}
 }
 
 func createContactPointServiceSut(t *testing.T, secretService secrets.Service) *ContactPointService {
 	// Encrypt secure settings.
 	cfg := createEncryptedConfig(t, secretService)
 	store := fakes.NewFakeAlertmanagerConfigStore(cfg)
+	return createContactPointServiceSutWithConfigStore(t, secretService, store)
+}
+
+func createContactPointServiceSutWithConfigStore(t *testing.T, secretService secrets.Service, configStore legacy_storage.AMConfigStore) *ContactPointService {
+	t.Helper()
+	// Encrypt secure settings.
 	xact := newNopTransactionManager()
 	provisioningStore := fakes.NewFakeProvisioningStore()
 
 	receiverService := notifier.NewReceiverService(
-		actest.FakeAccessControl{},
-		store,
+		ac.NewReceiverAccess[*models.Receiver](acimpl.ProvideAccessControl(featuremgmt.WithFeatures()), true),
+		legacy_storage.NewAlertmanagerConfigStore(configStore),
 		provisioningStore,
+		&fakeAlertRuleNotificationStore{},
 		secretService,
 		xact,
 		log.NewNopLogger(),
+		fakes.NewFakeReceiverPermissionsService(),
+		tracing.InitializeTracerForTest(),
 	)
 
-	return &ContactPointService{
-		configStore:       &alertmanagerConfigStoreImpl{store: store},
-		provenanceStore:   provisioningStore,
-		receiverService:   receiverService,
-		xact:              xact,
-		encryptionService: secretService,
-		log:               log.NewNopLogger(),
-	}
+	return NewContactPointService(
+		legacy_storage.NewAlertmanagerConfigStore(configStore),
+		secretService,
+		provisioningStore,
+		xact,
+		receiverService,
+		log.NewNopLogger(),
+		nil,
+		fakes.NewFakeReceiverPermissionsService(),
+	)
 }
 
 func createTestContactPoint() definitions.EmbeddedContactPoint {
@@ -418,9 +549,10 @@ func TestStitchReceivers(t *testing.T) {
 		name               string
 		initial            *definitions.PostableUserConfig
 		new                *definitions.PostableGrafanaReceiver
-		expModified        bool
 		expCfg             definitions.PostableApiAlertingConfig
-		expRenamedReceiver string
+		expOldReceiver     string
+		expCreatedReceiver bool
+		expFullRemoval     bool
 	}
 
 	cases := []testCase{
@@ -429,8 +561,8 @@ func TestStitchReceivers(t *testing.T) {
 			new: &definitions.PostableGrafanaReceiver{
 				UID: "does not exist",
 			},
-			expModified: false,
-			expCfg:      createTestConfigWithReceivers().AlertmanagerConfig,
+			expOldReceiver: "",
+			expCfg:         createTestConfigWithReceivers().AlertmanagerConfig,
 		},
 		{
 			name: "matching receiver with unchanged name, replaces",
@@ -439,7 +571,7 @@ func TestStitchReceivers(t *testing.T) {
 				Name: "receiver-2",
 				Type: "teams",
 			},
-			expModified: true,
+			expOldReceiver: "receiver-2",
 			expCfg: definitions.PostableApiAlertingConfig{
 				Config: definitions.Config{
 					Route: &definitions.Route{
@@ -500,15 +632,16 @@ func TestStitchReceivers(t *testing.T) {
 				Name: "new-receiver",
 				Type: "slack",
 			},
-			expModified:        true,
-			expRenamedReceiver: "new-receiver",
+			expOldReceiver:     "receiver-1",
+			expCreatedReceiver: true,
+			expFullRemoval:     true,
 			expCfg: definitions.PostableApiAlertingConfig{
 				Config: definitions.Config{
 					Route: &definitions.Route{
-						Receiver: "new-receiver",
+						Receiver: "receiver-1",
 						Routes: []*definitions.Route{
 							{
-								Receiver: "new-receiver",
+								Receiver: "receiver-1",
 							},
 						},
 					},
@@ -562,7 +695,8 @@ func TestStitchReceivers(t *testing.T) {
 				Name: "receiver-1",
 				Type: "slack",
 			},
-			expModified: true,
+			expOldReceiver:     "receiver-2",
+			expCreatedReceiver: false,
 			expCfg: definitions.PostableApiAlertingConfig{
 				Config: definitions.Config{
 					Route: &definitions.Route{
@@ -682,7 +816,8 @@ func TestStitchReceivers(t *testing.T) {
 				Name: "receiver-2",
 				Type: "slack",
 			},
-			expModified: true,
+			expOldReceiver:     "receiver-1",
+			expCreatedReceiver: false,
 			expCfg: definitions.PostableApiAlertingConfig{
 				Config: definitions.Config{
 					Route: &definitions.Route{
@@ -825,7 +960,8 @@ func TestStitchReceivers(t *testing.T) {
 				Name: "receiver-4",
 				Type: "slack",
 			},
-			expModified: true,
+			expOldReceiver:     "receiver-1",
+			expCreatedReceiver: false,
 			expCfg: definitions.PostableApiAlertingConfig{
 				Config: definitions.Config{
 					Route: &definitions.Route{
@@ -909,7 +1045,8 @@ func TestStitchReceivers(t *testing.T) {
 				Name: "brand-new-group",
 				Type: "opsgenie",
 			},
-			expModified: true,
+			expOldReceiver:     "receiver-2",
+			expCreatedReceiver: true,
 			expCfg: definitions.PostableApiAlertingConfig{
 				Config: definitions.Config{
 					Route: &definitions.Route{
@@ -980,7 +1117,8 @@ func TestStitchReceivers(t *testing.T) {
 				Name: "brand-new-group",
 				Type: "opsgenie",
 			},
-			expModified: true,
+			expOldReceiver:     "receiver-2", // Not the inconsistent receiver-3?
+			expCreatedReceiver: true,
 			expCfg: definitions.PostableApiAlertingConfig{
 				Config: definitions.Config{
 					Route: &definitions.Route{
@@ -1102,8 +1240,9 @@ func TestStitchReceivers(t *testing.T) {
 				Name: "receiver-1",
 				Type: "slack",
 			},
-			expModified:        true,
-			expRenamedReceiver: "receiver-1",
+			expOldReceiver:     "receiver-2",
+			expCreatedReceiver: false,
+			expFullRemoval:     true,
 			expCfg: definitions.PostableApiAlertingConfig{
 				Config: definitions.Config{
 					Route: &definitions.Route{
@@ -1113,7 +1252,7 @@ func TestStitchReceivers(t *testing.T) {
 								Receiver: "receiver-1",
 							},
 							{
-								Receiver: "receiver-1",
+								Receiver: "receiver-2",
 							},
 						},
 					},
@@ -1155,13 +1294,10 @@ func TestStitchReceivers(t *testing.T) {
 				cfg = c.initial
 			}
 
-			modified, renamedReceiver := stitchReceiver(cfg, c.new)
-			if c.expRenamedReceiver != "" {
-				assert.Equal(t, c.expRenamedReceiver, renamedReceiver)
-			} else {
-				assert.Empty(t, renamedReceiver)
-			}
-			require.Equal(t, c.expModified, modified)
+			renamedReceiver, fullRemoval, createdReceiver := stitchReceiver(cfg, c.new)
+			assert.Equalf(t, c.expOldReceiver, renamedReceiver, "expected old receiver to be %s, got %s", c.expOldReceiver, renamedReceiver)
+			assert.Equalf(t, c.expFullRemoval, fullRemoval, "expected full removal to be %t, got %t", c.expFullRemoval, fullRemoval)
+			assert.Equalf(t, c.expCreatedReceiver, createdReceiver, "expected created receiver to be %t, got %t", c.expCreatedReceiver, createdReceiver)
 			require.Equal(t, c.expCfg, cfg.AlertmanagerConfig)
 		})
 	}
