@@ -1,32 +1,36 @@
 package schedule
 
 import (
-	context "context"
+	"context"
 	"errors"
 	"fmt"
 	"net/url"
 	"time"
 
 	"github.com/benbjohnson/clock"
+	"github.com/prometheus/alertmanager/api/v2/models"
+	"go.opentelemetry.io/otel/attribute"
+	"go.opentelemetry.io/otel/codes"
+	"go.opentelemetry.io/otel/trace"
+
 	"github.com/grafana/grafana/pkg/infra/log"
 	"github.com/grafana/grafana/pkg/infra/tracing"
 	"github.com/grafana/grafana/pkg/services/datasources"
+	"github.com/grafana/grafana/pkg/services/ngalert/api/tooling/definitions"
 	"github.com/grafana/grafana/pkg/services/ngalert/eval"
 	"github.com/grafana/grafana/pkg/services/ngalert/metrics"
 	ngmodels "github.com/grafana/grafana/pkg/services/ngalert/models"
 	"github.com/grafana/grafana/pkg/services/ngalert/state"
 	"github.com/grafana/grafana/pkg/services/org"
 	"github.com/grafana/grafana/pkg/services/user"
+	"github.com/grafana/grafana/pkg/setting"
 	"github.com/grafana/grafana/pkg/util"
-	"go.opentelemetry.io/otel/attribute"
-	"go.opentelemetry.io/otel/codes"
-	"go.opentelemetry.io/otel/trace"
 )
 
 // Rule represents a single piece of work that is executed periodically by the ruler.
 type Rule interface {
 	// Run creates the resources that will perform the rule's work, and starts it. It blocks indefinitely, until Stop is called or another signal is sent.
-	Run(key ngmodels.AlertRuleKey) error
+	Run() error
 	// Stop shuts down the rule's execution with an optional reason. It has no effect if the rule has not yet been Run.
 	Stop(reason error)
 	// Eval sends a signal to execute the work represented by the rule, exactly one time.
@@ -34,12 +38,16 @@ type Rule interface {
 	Eval(eval *Evaluation) (bool, *Evaluation)
 	// Update sends a singal to change the definition of the rule.
 	Update(lastVersion RuleVersionAndPauseStatus) bool
+	// Type gives the type of the rule.
+	Type() ngmodels.RuleType
+	// Status indicates the status of the evaluating rule.
+	Status() ngmodels.RuleStatus
 }
 
-type ruleFactoryFunc func(context.Context) Rule
+type ruleFactoryFunc func(context.Context, *ngmodels.AlertRule) Rule
 
-func (f ruleFactoryFunc) new(ctx context.Context) Rule {
-	return f(ctx)
+func (f ruleFactoryFunc) new(ctx context.Context, rule *ngmodels.AlertRule) Rule {
+	return f(ctx, rule)
 }
 
 func newRuleFactory(
@@ -51,15 +59,34 @@ func newRuleFactory(
 	evalFactory eval.EvaluatorFactory,
 	ruleProvider ruleProvider,
 	clock clock.Clock,
+	rrCfg setting.RecordingRuleSettings,
 	met *metrics.Scheduler,
 	logger log.Logger,
 	tracer tracing.Tracer,
+	recordingWriter RecordingWriter,
 	evalAppliedHook evalAppliedFunc,
 	stopAppliedHook stopAppliedFunc,
 ) ruleFactoryFunc {
-	return func(ctx context.Context) Rule {
+	return func(ctx context.Context, rule *ngmodels.AlertRule) Rule {
+		if rule.Type() == ngmodels.RuleTypeRecording {
+			return newRecordingRule(
+				ctx,
+				rule.GetKey(),
+				maxAttempts,
+				clock,
+				evalFactory,
+				rrCfg,
+				logger,
+				met,
+				tracer,
+				recordingWriter,
+				evalAppliedHook,
+				stopAppliedHook,
+			)
+		}
 		return newAlertRule(
 			ctx,
+			rule.GetKeyWithGroup(),
 			appURL,
 			disableGrafanaFolder,
 			maxAttempts,
@@ -85,6 +112,8 @@ type ruleProvider interface {
 }
 
 type alertRule struct {
+	key ngmodels.AlertRuleKeyWithGroup
+
 	evalCh   chan *Evaluation
 	updateCh chan RuleVersionAndPauseStatus
 	ctx      context.Context
@@ -111,6 +140,7 @@ type alertRule struct {
 
 func newAlertRule(
 	parent context.Context,
+	key ngmodels.AlertRuleKeyWithGroup,
 	appURL *url.URL,
 	disableGrafanaFolder bool,
 	maxAttempts int64,
@@ -125,8 +155,9 @@ func newAlertRule(
 	evalAppliedHook func(ngmodels.AlertRuleKey, time.Time),
 	stopAppliedHook func(ngmodels.AlertRuleKey),
 ) *alertRule {
-	ctx, stop := util.WithCancelCause(parent)
+	ctx, stop := util.WithCancelCause(ngmodels.WithRuleKey(parent, key.AlertRuleKey))
 	return &alertRule{
+		key:                  key,
 		evalCh:               make(chan *Evaluation),
 		updateCh:             make(chan RuleVersionAndPauseStatus),
 		ctx:                  ctx,
@@ -142,11 +173,19 @@ func newAlertRule(
 		evalAppliedHook:      evalAppliedHook,
 		stopAppliedHook:      stopAppliedHook,
 		metrics:              met,
-		logger:               logger,
+		logger:               logger.FromContext(ctx),
 		tracer:               tracer,
 	}
 }
 
+func (a *alertRule) Type() ngmodels.RuleType {
+	return ngmodels.RuleTypeAlerting
+}
+
+func (a *alertRule) Status() ngmodels.RuleStatus {
+	return a.stateManager.GetStatusForRuleUID(a.key.OrgID, a.key.UID)
+}
+
 // eval signals the rule evaluation routine to perform the evaluation of the rule. Does nothing if the loop is stopped.
 // Before sending a message into the channel, it does non-blocking read to make sure that there is no concurrent send operation.
 // Returns a tuple where first element is
@@ -155,6 +194,11 @@ func newAlertRule(
 //
 // the second element contains a dropped message that was sent by a concurrent sender.
 func (a *alertRule) Eval(eval *Evaluation) (bool, *Evaluation) {
+	if a.key.AlertRuleKey != eval.rule.GetKey() {
+		// Make sure that rule has the same key. This should not happen
+		a.logger.Error("Invalid rule sent for evaluating. Skipping", "ruleKeyToEvaluate", eval.rule.GetKey().String())
+		return false, eval
+	}
 	// read the channel in unblocking manner to make sure that there is no concurrent send operation.
 	var droppedMsg *Evaluation
 	select {
@@ -195,57 +239,53 @@ func (a *alertRule) Stop(reason error) {
 	}
 }
 
-func (a *alertRule) Run(key ngmodels.AlertRuleKey) error {
-	grafanaCtx := ngmodels.WithRuleKey(a.ctx, key)
-	logger := a.logger.FromContext(grafanaCtx)
-	logger.Debug("Alert rule routine started")
+func (a *alertRule) Run() error {
+	grafanaCtx := a.ctx
+	a.logger.Debug("Alert rule routine started")
 
-	evalRunning := false
 	var currentFingerprint fingerprint
-	defer a.stopApplied(key)
+	defer a.stopApplied()
 	for {
 		select {
 		// used by external services (API) to notify that rule is updated.
 		case ctx := <-a.updateCh:
 			if currentFingerprint == ctx.Fingerprint {
-				logger.Info("Rule's fingerprint has not changed. Skip resetting the state", "currentFingerprint", currentFingerprint)
+				a.logger.Info("Rule's fingerprint has not changed. Skip resetting the state", "currentFingerprint", currentFingerprint)
 				continue
 			}
 
-			logger.Info("Clearing the state of the rule because it was updated", "isPaused", ctx.IsPaused, "fingerprint", ctx.Fingerprint)
+			a.logger.Info("Clearing the state of the rule because it was updated", "isPaused", ctx.IsPaused, "fingerprint", ctx.Fingerprint)
 			// clear the state. So the next evaluation will start from the scratch.
-			a.resetState(grafanaCtx, key, ctx.IsPaused)
+			a.resetState(grafanaCtx, ctx.IsPaused)
 			currentFingerprint = ctx.Fingerprint
 		// evalCh - used by the scheduler to signal that evaluation is needed.
 		case ctx, ok := <-a.evalCh:
 			if !ok {
-				logger.Debug("Evaluation channel has been closed. Exiting")
+				a.logger.Debug("Evaluation channel has been closed. Exiting")
 				return nil
 			}
-			if evalRunning {
-				continue
-			}
+			f := ctx.Fingerprint()
+			logger := a.logger.New("version", ctx.rule.Version, "fingerprint", f, "now", ctx.scheduledAt)
+			logger.Debug("Processing tick")
 
 			func() {
-				orgID := fmt.Sprint(key.OrgID)
+				orgID := fmt.Sprint(a.key.OrgID)
 				evalDuration := a.metrics.EvalDuration.WithLabelValues(orgID)
 				evalTotal := a.metrics.EvalTotal.WithLabelValues(orgID)
 
-				evalRunning = true
 				evalStart := a.clock.Now()
 				defer func() {
-					evalRunning = false
-					a.evalApplied(key, ctx.scheduledAt)
 					evalDuration.Observe(a.clock.Now().Sub(evalStart).Seconds())
+					a.evalApplied(ctx.scheduledAt)
 				}()
 
 				for attempt := int64(1); attempt <= a.maxAttempts; attempt++ {
 					isPaused := ctx.rule.IsPaused
-					f := ruleWithFolder{ctx.rule, ctx.folderTitle}.Fingerprint()
+
 					// Do not clean up state if the eval loop has just started.
 					var needReset bool
 					if currentFingerprint != 0 && currentFingerprint != f {
-						logger.Debug("Got a new version of alert rule. Clear up the state", "fingerprint", f)
+						logger.Debug("Got a new version of alert rule. Clear up the state", "current_fingerprint", currentFingerprint, "fingerprint", f)
 						needReset = true
 					}
 					// We need to reset state if the loop has started and the alert is already paused. It can happen,
@@ -253,7 +293,7 @@ func (a *alertRule) Run(key ngmodels.AlertRuleKey) error {
 					// lingers in DB and won't be cleaned up until next alert rule update.
 					needReset = needReset || (currentFingerprint == 0 && isPaused)
 					if needReset {
-						a.resetState(grafanaCtx, key, isPaused)
+						a.resetState(grafanaCtx, isPaused)
 					}
 					currentFingerprint = f
 					if isPaused {
@@ -275,6 +315,7 @@ func (a *alertRule) Run(key ngmodels.AlertRuleKey) error {
 						attribute.String("rule_fingerprint", fpStr),
 						attribute.String("tick", utcTick),
 					))
+					logger := logger.FromContext(tracingCtx)
 
 					// Check before any execution if the context was cancelled so that we don't do any evaluations.
 					if tracingCtx.Err() != nil {
@@ -283,20 +324,20 @@ func (a *alertRule) Run(key ngmodels.AlertRuleKey) error {
 						logger.Error("Skip evaluation and updating the state because the context has been cancelled", "version", ctx.rule.Version, "fingerprint", f, "attempt", attempt, "now", ctx.scheduledAt)
 						return
 					}
-
 					retry := attempt < a.maxAttempts
-					err := a.evaluate(tracingCtx, key, f, attempt, ctx, span, retry)
+					err := a.evaluate(tracingCtx, ctx, span, retry, logger)
 					// This is extremely confusing - when we exhaust all retry attempts, or we have no retryable errors
 					// we return nil - so technically, this is meaningless to know whether the evaluation has errors or not.
 					span.End()
 					if err == nil {
+						logger.Debug("Tick processed", "attempt", attempt, "duration", a.clock.Now().Sub(evalStart))
 						return
 					}
 
-					logger.Error("Failed to evaluate rule", "version", ctx.rule.Version, "fingerprint", f, "attempt", attempt, "now", ctx.scheduledAt, "error", err)
+					logger.Error("Failed to evaluate rule", "attempt", attempt, "error", err)
 					select {
 					case <-tracingCtx.Done():
-						logger.Error("Context has been cancelled while backing off", "version", ctx.rule.Version, "fingerprint", f, "attempt", attempt, "now", ctx.scheduledAt)
+						logger.Error("Context has been cancelled while backing off", "attempt", attempt)
 						return
 					case <-time.After(retryDelay):
 						continue
@@ -312,28 +353,27 @@ func (a *alertRule) Run(key ngmodels.AlertRuleKey) error {
 				// cases.
 				ctx, cancelFunc := context.WithTimeout(context.Background(), time.Minute)
 				defer cancelFunc()
-				states := a.stateManager.DeleteStateByRuleUID(ngmodels.WithRuleKey(ctx, key), key, ngmodels.StateReasonRuleDeleted)
-				a.notify(grafanaCtx, key, states)
+				states := a.stateManager.DeleteStateByRuleUID(ngmodels.WithRuleKey(ctx, a.key.AlertRuleKey), a.key, ngmodels.StateReasonRuleDeleted)
+				a.expireAndSend(grafanaCtx, states)
 			}
-			logger.Debug("Stopping alert rule routine")
+			a.logger.Debug("Stopping alert rule routine")
 			return nil
 		}
 	}
 }
 
-func (a *alertRule) evaluate(ctx context.Context, key ngmodels.AlertRuleKey, f fingerprint, attempt int64, e *Evaluation, span trace.Span, retry bool) error {
-	orgID := fmt.Sprint(key.OrgID)
+func (a *alertRule) evaluate(ctx context.Context, e *Evaluation, span trace.Span, retry bool, logger log.Logger) error {
+	orgID := fmt.Sprint(a.key.OrgID)
 	evalAttemptTotal := a.metrics.EvalAttemptTotal.WithLabelValues(orgID)
 	evalAttemptFailures := a.metrics.EvalAttemptFailures.WithLabelValues(orgID)
 	evalTotalFailures := a.metrics.EvalFailures.WithLabelValues(orgID)
 	processDuration := a.metrics.ProcessDuration.WithLabelValues(orgID)
 	sendDuration := a.metrics.SendDuration.WithLabelValues(orgID)
 
-	logger := a.logger.FromContext(ctx).New("version", e.rule.Version, "fingerprint", f, "attempt", attempt, "now", e.scheduledAt).FromContext(ctx)
 	start := a.clock.Now()
 
 	evalCtx := eval.NewContextWithPreviousResults(ctx, SchedulerUserFor(e.rule.OrgID), a.newLoadedMetricsReader(e.rule))
-	ruleEval, err := a.evalFactory.Create(evalCtx, e.rule.GetEvalCondition())
+	ruleEval, err := a.evalFactory.Create(evalCtx, e.rule.GetEvalCondition().WithSource("scheduler").WithFolder(e.folderTitle))
 	var results eval.Results
 	var dur time.Duration
 	if err != nil {
@@ -389,71 +429,84 @@ func (a *alertRule) evaluate(ctx context.Context, key ngmodels.AlertRuleKey, f f
 			err = results.Error()
 		}
 
+		logger.Debug("Alert rule evaluated", "error", err, "duration", dur)
 		span.SetStatus(codes.Error, "rule evaluation failed")
 		span.RecordError(err)
 	} else {
-		logger.Debug("Alert rule evaluated", "results", results, "duration", dur)
+		logger.Debug("Alert rule evaluated", "results", len(results), "duration", dur)
 		span.AddEvent("rule evaluated", trace.WithAttributes(
 			attribute.Int64("results", int64(len(results))),
 		))
 	}
 	start = a.clock.Now()
-	processedStates := a.stateManager.ProcessEvalResults(
+	_ = a.stateManager.ProcessEvalResults(
 		ctx,
 		e.scheduledAt,
 		e.rule,
 		results,
 		state.GetRuleExtraLabels(logger, e.rule, e.folderTitle, !a.disableGrafanaFolder),
+		func(ctx context.Context, statesToSend state.StateTransitions) {
+			start := a.clock.Now()
+			alerts := a.send(ctx, logger, statesToSend)
+			span.AddEvent("results sent", trace.WithAttributes(
+				attribute.Int64("alerts_sent", int64(len(alerts.PostableAlerts))),
+			))
+			sendDuration.Observe(a.clock.Now().Sub(start).Seconds())
+		},
 	)
 	processDuration.Observe(a.clock.Now().Sub(start).Seconds())
 
-	start = a.clock.Now()
-	alerts := state.FromStateTransitionToPostableAlerts(processedStates, a.stateManager, a.appURL)
-	span.AddEvent("results processed", trace.WithAttributes(
-		attribute.Int64("state_transitions", int64(len(processedStates))),
-		attribute.Int64("alerts_to_send", int64(len(alerts.PostableAlerts))),
-	))
-	if len(alerts.PostableAlerts) > 0 {
-		a.sender.Send(ctx, key, alerts)
-	}
-	sendDuration.Observe(a.clock.Now().Sub(start).Seconds())
-
 	return nil
 }
 
-func (a *alertRule) notify(ctx context.Context, key ngmodels.AlertRuleKey, states []state.StateTransition) {
+// send sends alerts for the given state transitions.
+func (a *alertRule) send(ctx context.Context, logger log.Logger, states state.StateTransitions) definitions.PostableAlerts {
+	alerts := definitions.PostableAlerts{PostableAlerts: make([]models.PostableAlert, 0, len(states))}
+	for _, alertState := range states {
+		alerts.PostableAlerts = append(alerts.PostableAlerts, *state.StateToPostableAlert(alertState, a.appURL))
+	}
+
+	if len(alerts.PostableAlerts) > 0 {
+		logger.Debug("Sending transitions to notifier", "transitions", len(alerts.PostableAlerts))
+		a.sender.Send(ctx, a.key.AlertRuleKey, alerts)
+	}
+	return alerts
+}
+
+// sendExpire sends alerts to expire all previously firing alerts in the provided state transitions.
+func (a *alertRule) expireAndSend(ctx context.Context, states []state.StateTransition) {
 	expiredAlerts := state.FromAlertsStateToStoppedAlert(states, a.appURL, a.clock)
 	if len(expiredAlerts.PostableAlerts) > 0 {
-		a.sender.Send(ctx, key, expiredAlerts)
+		a.sender.Send(ctx, a.key.AlertRuleKey, expiredAlerts)
 	}
 }
 
-func (a *alertRule) resetState(ctx context.Context, key ngmodels.AlertRuleKey, isPaused bool) {
-	rule := a.ruleProvider.get(key)
+func (a *alertRule) resetState(ctx context.Context, isPaused bool) {
+	rule := a.ruleProvider.get(a.key.AlertRuleKey)
 	reason := ngmodels.StateReasonUpdated
 	if isPaused {
 		reason = ngmodels.StateReasonPaused
 	}
 	states := a.stateManager.ResetStateByRuleUID(ctx, rule, reason)
-	a.notify(ctx, key, states)
+	a.expireAndSend(ctx, states)
 }
 
 // evalApplied is only used on tests.
-func (a *alertRule) evalApplied(alertDefKey ngmodels.AlertRuleKey, now time.Time) {
+func (a *alertRule) evalApplied(now time.Time) {
 	if a.evalAppliedHook == nil {
 		return
 	}
 
-	a.evalAppliedHook(alertDefKey, now)
+	a.evalAppliedHook(a.key.AlertRuleKey, now)
 }
 
 // stopApplied is only used on tests.
-func (a *alertRule) stopApplied(alertDefKey ngmodels.AlertRuleKey) {
+func (a *alertRule) stopApplied() {
 	if a.stopAppliedHook == nil {
 		return
 	}
 
-	a.stopAppliedHook(alertDefKey)
+	a.stopAppliedHook(a.key.AlertRuleKey)
 }
 
 func SchedulerUserFor(orgID int64) *user.SignedInUser {
@@ -468,6 +521,9 @@ func SchedulerUserFor(orgID int64) *user.SignedInUser {
 				datasources.ActionQuery: []string{
 					datasources.ScopeAll,
 				},
+				datasources.ActionRead: []string{
+					datasources.ScopeAll,
+				},
 			},
 		},
 	}
