@@ -5,15 +5,21 @@ import (
 	"encoding/json"
 	"fmt"
 	"math"
+	"regexp"
 	"sort"
+	"strconv"
+	"strings"
 	"time"
 
 	"github.com/benbjohnson/clock"
 	"github.com/grafana/grafana-plugin-sdk-go/data"
 	"go.opentelemetry.io/otel/trace"
 
+	"github.com/grafana/grafana/pkg/apimachinery/errutil"
 	"github.com/grafana/grafana/pkg/components/simplejson"
 	"github.com/grafana/grafana/pkg/infra/log"
+	"github.com/grafana/grafana/pkg/infra/tracing"
+	"github.com/grafana/grafana/pkg/services/ngalert/accesscontrol"
 	"github.com/grafana/grafana/pkg/services/ngalert/client"
 	"github.com/grafana/grafana/pkg/services/ngalert/eval"
 	"github.com/grafana/grafana/pkg/services/ngalert/metrics"
@@ -40,10 +46,30 @@ const (
 
 const defaultQueryRange = 6 * time.Hour
 
+var (
+	ErrLokiQueryTooLong = errutil.BadRequest("loki.requestTooLong").MustTemplate(
+		"Request to Loki exceeded ({{.Public.QuerySize}} bytes) configured maximum size of {{.Public.MaxLimit}} bytes. Query: {{.Private.Query}}",
+		errutil.WithPublic("Query for Loki exceeded the configured limit of {{.Public.MaxLimit}} bytes. Remove some filters and try again."),
+	)
+)
+
+func NewErrLokiQueryTooLong(query string, maxLimit int) error {
+	return ErrLokiQueryTooLong.Build(errutil.TemplateData{
+		Private: map[string]any{
+			"Query": query,
+		},
+		Public: map[string]any{
+			"MaxLimit":  maxLimit,
+			"QuerySize": len(query),
+		},
+	})
+}
+
 type remoteLokiClient interface {
 	Ping(context.Context) error
 	Push(context.Context, []Stream) error
 	RangeQuery(ctx context.Context, logQL string, start, end, limit int64) (QueryRes, error)
+	MaxQuerySize() int
 }
 
 // RemoteLokibackend is a state.Historian that records state history to an external Loki instance.
@@ -53,16 +79,19 @@ type RemoteLokiBackend struct {
 	clock          clock.Clock
 	metrics        *metrics.Historian
 	log            log.Logger
+	ac             AccessControl
+	ruleStore      RuleStore
 }
 
-func NewRemoteLokiBackend(cfg LokiConfig, req client.Requester, metrics *metrics.Historian) *RemoteLokiBackend {
-	logger := log.New("ngalert.state.historian", "backend", "loki")
+func NewRemoteLokiBackend(logger log.Logger, cfg LokiConfig, req client.Requester, metrics *metrics.Historian, tracer tracing.Tracer, ruleStore RuleStore, ac AccessControl) *RemoteLokiBackend {
 	return &RemoteLokiBackend{
-		client:         NewLokiClient(cfg, req, metrics, logger),
+		client:         NewLokiClient(cfg, req, metrics, logger, tracer),
 		externalLabels: cfg.ExternalLabels,
 		clock:          clock.New(),
 		metrics:        metrics,
 		log:            logger,
+		ac:             ac,
+		ruleStore:      ruleStore,
 	}
 }
 
@@ -95,12 +124,12 @@ func (h *RemoteLokiBackend) Record(ctx context.Context, rule history_model.RuleM
 		defer cancel()
 		defer close(errCh)
 		logger := h.log.FromContext(ctx)
-
+		logger.Debug("Saving state history batch", "samples", len(logStream.Values))
 		org := fmt.Sprint(rule.OrgID)
 		h.metrics.WritesTotal.WithLabelValues(org, "loki").Inc()
 		h.metrics.TransitionsTotal.WithLabelValues(org).Add(float64(len(logStream.Values)))
 
-		if err := h.recordStreams(ctx, []Stream{logStream}, logger); err != nil {
+		if err := h.recordStreams(ctx, logStream, logger); err != nil {
 			logger.Error("Failed to save alert state history batch", "error", err)
 			h.metrics.WritesFailed.WithLabelValues(org, "loki").Inc()
 			h.metrics.TransitionsFailed.WithLabelValues(org).Add(float64(len(logStream.Values)))
@@ -112,11 +141,19 @@ func (h *RemoteLokiBackend) Record(ctx context.Context, rule history_model.RuleM
 
 // Query retrieves state history entries from an external Loki instance and formats the results into a dataframe.
 func (h *RemoteLokiBackend) Query(ctx context.Context, query models.HistoryQuery) (*data.Frame, error) {
-	logQL, err := BuildLogQuery(query)
+	uids, err := h.getFolderUIDsForFilter(ctx, query)
 	if err != nil {
 		return nil, err
 	}
 
+	queries, err := BuildLogQuery(query, uids, h.client.MaxQuerySize())
+	if err != nil {
+		return nil, err
+	}
+	if len(queries) > 1 {
+		h.log.FromContext(ctx).Info("Execute query in multiple batches", "batchSize", len(queries), "folders", len(uids), "maxQueryLimit", h.client.MaxQuerySize())
+	}
+
 	now := time.Now().UTC()
 	if query.To.IsZero() {
 		query.To = now
@@ -124,41 +161,30 @@ func (h *RemoteLokiBackend) Query(ctx context.Context, query models.HistoryQuery
 	if query.From.IsZero() {
 		query.From = now.Add(-defaultQueryRange)
 	}
-
-	// Timestamps are expected in RFC3339Nano.
-	res, err := h.client.RangeQuery(ctx, logQL, query.From.UnixNano(), query.To.UnixNano(), int64(query.Limit))
-	if err != nil {
-		return nil, err
+	var res []Stream
+	for _, logQL := range queries {
+		// Timestamps are expected in RFC3339Nano.
+		// Apply user-defined limit to every request. Multiple batches is a very rare case, and therefore we can tolerate getting more data than needed.
+		// The limit can be applied after all results are merged
+		r, err := h.client.RangeQuery(ctx, logQL, query.From.UnixNano(), query.To.UnixNano(), int64(query.Limit))
+		if err != nil {
+			return nil, err
+		}
+		res = append(res, r.Data.Result...)
 	}
-	return merge(res, query.RuleUID)
-}
-
-func buildSelectors(query models.HistoryQuery) ([]Selector, error) {
-	// OrgID and the state history label are static and will be included in all queries.
-	selectors := make([]Selector, 2)
-
-	// Set the predefined selector orgID.
-	selector, err := NewSelector(OrgIDLabel, "=", fmt.Sprintf("%d", query.OrgID))
-	if err != nil {
-		return nil, err
-	}
-	selectors[0] = selector
-
-	// Set the predefined selector for the state history label.
-	selector, err = NewSelector(StateHistoryLabelKey, "=", StateHistoryLabelValue)
-	if err != nil {
-		return nil, err
-	}
-	selectors[1] = selector
-
-	return selectors, nil
+	return merge(res, uids)
 }
 
 // merge will put all the results in one array sorted by timestamp.
-func merge(res QueryRes, ruleUID string) (*data.Frame, error) {
+func merge(res []Stream, folderUIDToFilter []string) (*data.Frame, error) {
+	filterByFolderUIDMap := make(map[string]struct{}, len(folderUIDToFilter))
+	for _, uid := range folderUIDToFilter {
+		filterByFolderUIDMap[uid] = struct{}{}
+	}
+
 	// Find the total number of elements in all arrays.
 	totalLen := 0
-	for _, arr := range res.Data.Result {
+	for _, arr := range res {
 		totalLen += len(arr.Values)
 	}
 
@@ -178,17 +204,29 @@ func merge(res QueryRes, ruleUID string) (*data.Frame, error) {
 	labels := make([]json.RawMessage, 0, totalLen)
 
 	// Initialize a slice of pointers to the current position in each array.
-	pointers := make([]int, len(res.Data.Result))
+	pointers := make([]int, len(res))
 	for {
 		minTime := int64(math.MaxInt64)
 		minEl := Sample{}
 		minElStreamIdx := -1
 		// Find the element with the earliest time among all arrays.
-		for i, stream := range res.Data.Result {
+		for i, stream := range res {
 			// Skip if we already reached the end of the current array.
 			if len(stream.Values) == pointers[i] {
 				continue
 			}
+			// check if stream should be in the results
+			if len(filterByFolderUIDMap) > 0 {
+				folderLbl, ok := stream.Stream[FolderUIDLabel]
+				if !ok {
+					continue // skip entries without folder UID, only if needs filtering
+				}
+				_, ok = filterByFolderUIDMap[folderLbl]
+				if !ok {
+					continue
+				}
+			}
+
 			curTime := stream.Values[pointers[i]].T.UnixNano()
 			if pointers[i] < len(stream.Values) && curTime < minTime {
 				minTime = curTime
@@ -208,7 +246,7 @@ func merge(res QueryRes, ruleUID string) (*data.Frame, error) {
 		// Append the minimum element to the merged slice and move the pointer.
 		tsNano := minEl.T.UnixNano()
 		// TODO: In general, perhaps we should omit the offending line and log, rather than failing the request entirely.
-		streamLbls := res.Data.Result[minElStreamIdx].Stream
+		streamLbls := res[minElStreamIdx].Stream
 		lblsJson, err := json.Marshal(streamLbls)
 		if err != nil {
 			return nil, fmt.Errorf("failed to serialize stream labels: %w", err)
@@ -283,12 +321,12 @@ func StatesToStream(rule history_model.RuleMeta, states []state.StateTransition,
 	}
 }
 
-func (h *RemoteLokiBackend) recordStreams(ctx context.Context, streams []Stream, logger log.Logger) error {
-	if err := h.client.Push(ctx, streams); err != nil {
+func (h *RemoteLokiBackend) recordStreams(ctx context.Context, stream Stream, logger log.Logger) error {
+	if err := h.client.Push(ctx, []Stream{stream}); err != nil {
 		return err
 	}
 
-	logger.Debug("Done saving alert state history batch")
+	logger.Debug("Done saving alert state history batch", "samples", len(stream.Values))
 	return nil
 }
 
@@ -329,78 +367,122 @@ func jsonifyRow(line string) (json.RawMessage, error) {
 	return json.Marshal(entry)
 }
 
-type Selector struct {
-	// Label to Select
-	Label string
-	Op    Operator
-	// Value that is expected
-	Value string
-}
-
-func NewSelector(label, op, value string) (Selector, error) {
-	if !isValidOperator(op) {
-		return Selector{}, fmt.Errorf("'%s' is not a valid query operator", op)
-	}
-	return Selector{Label: label, Op: Operator(op), Value: value}, nil
-}
-
-func selectorString(selectors []Selector) string {
-	if len(selectors) == 0 {
-		return "{}"
-	}
-	// Build the query selector.
-	query := ""
-	for _, s := range selectors {
-		query += fmt.Sprintf("%s%s%q,", s.Label, s.Op, s.Value)
-	}
-	// Remove the last comma, as we append one to every selector.
-	query = query[:len(query)-1]
-	return "{" + query + "}"
-}
-
-func isValidOperator(op string) bool {
-	switch op {
-	case "=", "!=", "=~", "!~":
-		return true
-	}
-	return false
-}
-
-func BuildLogQuery(query models.HistoryQuery) (string, error) {
-	selectors, err := buildSelectors(query)
+// BuildLogQuery converts models.HistoryQuery and a list of folder UIDs to Loki queries.
+// It can return multiple queries if the list of folder UIDs is too big to fit into single query.
+// If there is a folder UID long enough to exceed a query size it returns ErrQueryTooLong.
+func BuildLogQuery(query models.HistoryQuery, folderUIDs []string, maxQuerySize int) ([]string, error) {
+	// first build tail of the query (if exists) to know what remaining capacity we have for folders
+	tail, err := buildQueryTail(query)
 	if err != nil {
-		return "", fmt.Errorf("failed to build the provided selectors: %w", err)
+		return nil, err
+	}
+	// build the base selectors. skip the closing bracket because we will append folders below. Closing bracket will be added at the end
+	head := fmt.Sprintf(`{%s="%d",%s=%q`, OrgIDLabel, query.OrgID, StateHistoryLabelKey, StateHistoryLabelValue)
+
+	// check if system-defined + user-defined query parameters do not exceed maximum size
+	baseQuerySize := len(head) + 1 + len(tail) // 1 stands for closing bracket
+	if len(head)+1+len(tail) > maxQuerySize {
+		return nil, NewErrLokiQueryTooLong(head+"}"+tail, maxQuerySize)
 	}
 
-	logQL := selectorString(selectors)
-
-	if queryHasLogFilters(query) {
-		logQL = fmt.Sprintf("%s | json", logQL)
+	// nothing to append to the head. Construct the query and return what we have
+	if len(folderUIDs) == 0 {
+		return []string{head + "}" + tail}, nil
 	}
+	remainingFolders := folderUIDs
+	var result []string
+	for len(remainingFolders) > 0 { // iterating until all folders are processed and at least one result
+		// pre-calculate the size of the string to avoid excessive re-allocations
+		requiredLengthEstimate := baseQuerySize + 1 + len(FolderUIDLabel) + 3 // 1 - comma, 3 - operator + opening backtick
+		for _, folder := range remainingFolders {                             // this does not account to escaping characters because it's not expected generally
+			requiredLengthEstimate += len(folder) + 1 // 1 - accounts to pipe-separator and  backtick at the end
+			if requiredLengthEstimate > maxQuerySize {
+				break
+			}
+		}
+		s := strings.Builder{}
+		s.Grow(requiredLengthEstimate)
+		s.WriteString(head)
+		s.WriteString(",")
+		s.WriteString(FolderUIDLabel)
+		s.WriteString("=~`")
+		added := false
+		for len(remainingFolders) > 0 {
+			uid := regexp.QuoteMeta(remainingFolders[0])
+			requiredLength := s.Len() + len(tail) + len(uid) + 2 // 2 - backtick + closing bracket
+			if added {
+				requiredLength++ // account to pipe symbol
+			}
+			if requiredLength > maxQuerySize {
+				if !added {
+					// finish the query for logging
+					s.WriteString(uid)
+					s.WriteString("`}")
+					s.WriteString(tail)
+					return nil, NewErrLokiQueryTooLong(s.String(), maxQuerySize)
+				}
+				break
+			}
+			if added {
+				s.WriteString("|")
+			}
+			s.WriteString(uid)
+			added = true
+			remainingFolders = remainingFolders[1:]
+		}
+		s.WriteString("`")
+		s.WriteString("}")
+		s.WriteString(tail)
+		result = append(result, s.String())
+	}
+	return result, nil
+}
+
+func buildQueryTail(query models.HistoryQuery) (string, error) {
+	if !queryHasLogFilters(query) {
+		return "", nil
+	}
+	b := strings.Builder{}
+	b.WriteString(" | json")
 
 	if query.RuleUID != "" {
-		logQL = fmt.Sprintf("%s | ruleUID=%q", logQL, query.RuleUID)
+		b.WriteString(" | ruleUID=")
+		_, err := fmt.Fprintf(&b, "%q", query.RuleUID)
+		if err != nil {
+			return "", err
+		}
 	}
 	if query.DashboardUID != "" {
-		logQL = fmt.Sprintf("%s | dashboardUID=%q", logQL, query.DashboardUID)
+		b.WriteString(" | dashboardUID=")
+		_, err := fmt.Fprintf(&b, "%q", query.DashboardUID)
+		if err != nil {
+			return "", err
+		}
 	}
 	if query.PanelID != 0 {
-		logQL = fmt.Sprintf("%s | panelID=%d", logQL, query.PanelID)
+		b.WriteString(" | panelID=")
+		b.WriteString(strconv.FormatInt(query.PanelID, 10))
 	}
 
-	labelFilters := ""
+	requiredSize := 0
 	labelKeys := make([]string, 0, len(query.Labels))
-	for k := range query.Labels {
+	for k, v := range query.Labels {
+		requiredSize += len(k) + len(v) + 13 // 13 all literals below
 		labelKeys = append(labelKeys, k)
 	}
 	// Ensure that all queries we build are deterministic.
 	sort.Strings(labelKeys)
+	b.Grow(requiredSize)
 	for _, k := range labelKeys {
-		labelFilters += fmt.Sprintf(" | labels_%s=%q", k, query.Labels[k])
+		b.WriteString(" | labels_")
+		b.WriteString(k)
+		b.WriteString("=")
+		_, err := fmt.Fprintf(&b, "%q", query.Labels[k])
+		if err != nil {
+			return "", err
+		}
 	}
-	logQL += labelFilters
-
-	return logQL, nil
+	return b.String(), nil
 }
 
 func queryHasLogFilters(query models.HistoryQuery) bool {
@@ -409,3 +491,49 @@ func queryHasLogFilters(query models.HistoryQuery) bool {
 		query.PanelID != 0 ||
 		len(query.Labels) > 0
 }
+
+func (h *RemoteLokiBackend) getFolderUIDsForFilter(ctx context.Context, query models.HistoryQuery) ([]string, error) {
+	bypass, err := h.ac.CanReadAllRules(ctx, query.SignedInUser)
+	if err != nil {
+		return nil, err
+	}
+	if bypass { // if user has access to all rules and folder, remove filter
+		return nil, nil
+	}
+	// if there is a filter by rule UID, find that rule UID and make sure that user has access to it.
+	if query.RuleUID != "" {
+		rule, err := h.ruleStore.GetAlertRuleByUID(ctx, &models.GetAlertRuleByUIDQuery{
+			UID:   query.RuleUID,
+			OrgID: query.OrgID,
+		})
+		if err != nil {
+			return nil, fmt.Errorf("failed to fetch alert rule by UID: %w", err)
+		}
+		if rule == nil {
+			return nil, models.ErrAlertRuleNotFound
+		}
+		return nil, h.ac.AuthorizeAccessInFolder(ctx, query.SignedInUser, rule)
+	}
+	// if no filter, then we need to get all namespaces user has access to
+	folders, err := h.ruleStore.GetUserVisibleNamespaces(ctx, query.OrgID, query.SignedInUser)
+	if err != nil {
+		return nil, fmt.Errorf("failed to fetch folders that user can access: %w", err)
+	}
+	uids := make([]string, 0, len(folders))
+	// now keep only UIDs of folder in which user can read rules.
+	for _, f := range folders {
+		hasAccess, err := h.ac.HasAccessInFolder(ctx, query.SignedInUser, models.Namespace(*f))
+		if err != nil {
+			return nil, err
+		}
+		if !hasAccess {
+			continue
+		}
+		uids = append(uids, f.UID)
+	}
+	if len(uids) == 0 {
+		return nil, accesscontrol.NewAuthorizationErrorGeneric("read rules in any folder")
+	}
+	sort.Strings(uids)
+	return uids, nil
+}
