@@ -63,7 +63,7 @@ func CalculateChanges(ctx context.Context, ruleReader RuleReader, groupKey model
 	q := &models.ListAlertRulesQuery{
 		OrgID:         groupKey.OrgID,
 		NamespaceUIDs: []string{groupKey.NamespaceUID},
-		RuleGroup:     groupKey.RuleGroup,
+		RuleGroups:    []string{groupKey.RuleGroup},
 	}
 	existingGroupRules, err := ruleReader.ListAlertRules(ctx, q)
 	if err != nil {
@@ -113,10 +113,9 @@ func calculateChanges(ctx context.Context, ruleReader RuleReader, groupKey model
 					}
 					loadedRulesByUID[rule.UID] = rule
 				}
-				if existing == nil {
-					return nil, fmt.Errorf("failed to update rule with UID %s because %w", r.UID, models.ErrAlertRuleNotFound)
+				if existing != nil {
+					affectedGroups[existing.GetGroupKey()] = ruleList
 				}
-				affectedGroups[existing.GetGroupKey()] = ruleList
 			}
 		}
 
@@ -126,18 +125,14 @@ func calculateChanges(ctx context.Context, ruleReader RuleReader, groupKey model
 		}
 
 		models.PatchPartialAlertRule(existing, r)
-
 		diff := existing.Diff(&r.AlertRule, AlertRuleFieldsToIgnoreInDiff[:]...)
-		if len(diff) == 0 {
-			continue
+		if len(diff) > 0 {
+			toUpdate = append(toUpdate, RuleDelta{
+				Existing: existing,
+				New:      &r.AlertRule,
+				Diff:     diff,
+			})
 		}
-
-		toUpdate = append(toUpdate, RuleDelta{
-			Existing: existing,
-			New:      &r.AlertRule,
-			Diff:     diff,
-		})
-		continue
 	}
 
 	toDelete := make([]*models.AlertRule, 0, len(existingGroupRulesUIDs))
@@ -203,7 +198,7 @@ func CalculateRuleUpdate(ctx context.Context, ruleReader RuleReader, rule *model
 	q := &models.ListAlertRulesQuery{
 		OrgID:         rule.OrgID,
 		NamespaceUIDs: []string{rule.NamespaceUID},
-		RuleGroup:     rule.RuleGroup,
+		RuleGroups:    []string{rule.RuleGroup},
 	}
 	existingGroupRules, err := ruleReader.ListAlertRules(ctx, q)
 	if err != nil {
@@ -232,7 +227,7 @@ func CalculateRuleGroupDelete(ctx context.Context, ruleReader RuleReader, groupK
 	q := models.ListAlertRulesQuery{
 		OrgID:         groupKey.OrgID,
 		NamespaceUIDs: []string{groupKey.NamespaceUID},
-		RuleGroup:     groupKey.RuleGroup,
+		RuleGroups:    []string{groupKey.RuleGroup},
 	}
 	ruleList, err := ruleReader.ListAlertRules(ctx, &q)
 	if err != nil {
@@ -288,7 +283,7 @@ func CalculateRuleCreate(ctx context.Context, ruleReader RuleReader, rule *model
 	q := &models.ListAlertRulesQuery{
 		OrgID:         rule.OrgID,
 		NamespaceUIDs: []string{rule.NamespaceUID},
-		RuleGroup:     rule.RuleGroup,
+		RuleGroups:    []string{rule.RuleGroup},
 	}
 	group, err := ruleReader.ListAlertRules(ctx, q)
 	if err != nil {
