@@ -7,7 +7,9 @@ import (
 	"github.com/stretchr/testify/require"
 
 	"github.com/grafana/grafana-azure-sdk-go/v2/azsettings"
+	"github.com/grafana/grafana-plugin-sdk-go/backend"
 
+	"github.com/grafana/grafana/pkg/plugins/auth"
 	"github.com/grafana/grafana/pkg/services/featuremgmt"
 	"github.com/grafana/grafana/pkg/setting"
 )
@@ -22,7 +24,7 @@ func TestRequestConfigProvider_PluginRequestConfig_Defaults(t *testing.T) {
 		"GF_SQL_MAX_OPEN_CONNS_DEFAULT":            "0",
 		"GF_SQL_MAX_IDLE_CONNS_DEFAULT":            "0",
 		"GF_SQL_MAX_CONN_LIFETIME_SECONDS_DEFAULT": "0",
-	}, p.PluginRequestConfig(context.Background(), ""))
+	}, p.PluginRequestConfig(context.Background(), "", nil))
 }
 
 func TestRequestConfigProvider_PluginRequestConfig(t *testing.T) {
@@ -132,7 +134,7 @@ func TestRequestConfigProvider_PluginRequestConfig(t *testing.T) {
 	for _, tc := range tcs {
 		t.Run(tc.name, func(t *testing.T) {
 			p := NewRequestConfigProvider(tc.cfg)
-			require.Subset(t, p.PluginRequestConfig(context.Background(), ""), tc.expected)
+			require.Subset(t, p.PluginRequestConfig(context.Background(), "", nil), tc.expected)
 		})
 	}
 }
@@ -167,7 +169,7 @@ func TestRequestConfigProvider_PluginRequestConfig_featureToggles(t *testing.T)
 			require.NoError(t, err)
 
 			p := NewRequestConfigProvider(pCfg)
-			require.Subset(t, p.PluginRequestConfig(context.Background(), ""), tc.expectedConfig)
+			require.Subset(t, p.PluginRequestConfig(context.Background(), "", nil), tc.expectedConfig)
 		}
 	})
 }
@@ -181,7 +183,7 @@ func TestRequestConfigProvider_PluginRequestConfig_appURL(t *testing.T) {
 		require.NoError(t, err)
 
 		p := NewRequestConfigProvider(pCfg)
-		require.Subset(t, p.PluginRequestConfig(context.Background(), ""), map[string]string{"GF_APP_URL": "https://myorg.com/"})
+		require.Subset(t, p.PluginRequestConfig(context.Background(), "", nil), map[string]string{"GF_APP_URL": "https://myorg.com/"})
 	})
 }
 
@@ -197,7 +199,7 @@ func TestRequestConfigProvider_PluginRequestConfig_SQL(t *testing.T) {
 		require.NoError(t, err)
 
 		p := NewRequestConfigProvider(pCfg)
-		require.Subset(t, p.PluginRequestConfig(context.Background(), ""), map[string]string{
+		require.Subset(t, p.PluginRequestConfig(context.Background(), "", nil), map[string]string{
 			"GF_SQL_ROW_LIMIT":                         "23",
 			"GF_SQL_MAX_OPEN_CONNS_DEFAULT":            "24",
 			"GF_SQL_MAX_IDLE_CONNS_DEFAULT":            "25",
@@ -219,7 +221,7 @@ func TestRequestConfigProvider_PluginRequestConfig_SQL(t *testing.T) {
 			"GF_SQL_MAX_OPEN_CONNS_DEFAULT":            "0",
 			"GF_SQL_MAX_IDLE_CONNS_DEFAULT":            "0",
 			"GF_SQL_MAX_CONN_LIFETIME_SECONDS_DEFAULT": "0",
-		}, p.PluginRequestConfig(context.Background(), ""))
+		}, p.PluginRequestConfig(context.Background(), "", nil))
 	})
 }
 
@@ -232,7 +234,7 @@ func TestRequestConfigProvider_PluginRequestConfig_concurrentQueryCount(t *testi
 		require.NoError(t, err)
 
 		p := NewRequestConfigProvider(pCfg)
-		require.Subset(t, p.PluginRequestConfig(context.Background(), ""), map[string]string{"GF_CONCURRENT_QUERY_COUNT": "42"})
+		require.Subset(t, p.PluginRequestConfig(context.Background(), "", nil), map[string]string{"GF_CONCURRENT_QUERY_COUNT": "42"})
 	})
 
 	t.Run("Doesn't set the concurrent query count if it is not in the config", func(t *testing.T) {
@@ -241,7 +243,7 @@ func TestRequestConfigProvider_PluginRequestConfig_concurrentQueryCount(t *testi
 		require.NoError(t, err)
 
 		p := NewRequestConfigProvider(pCfg)
-		require.NotContains(t, p.PluginRequestConfig(context.Background(), ""), "GF_CONCURRENT_QUERY_COUNT")
+		require.NotContains(t, p.PluginRequestConfig(context.Background(), "", nil), "GF_CONCURRENT_QUERY_COUNT")
 	})
 
 	t.Run("Doesn't set the concurrent query count if it is zero", func(t *testing.T) {
@@ -252,7 +254,7 @@ func TestRequestConfigProvider_PluginRequestConfig_concurrentQueryCount(t *testi
 		require.NoError(t, err)
 
 		p := NewRequestConfigProvider(pCfg)
-		require.NotContains(t, p.PluginRequestConfig(context.Background(), ""), "GF_CONCURRENT_QUERY_COUNT")
+		require.NotContains(t, p.PluginRequestConfig(context.Background(), "", nil), "GF_CONCURRENT_QUERY_COUNT")
 	})
 }
 
@@ -264,7 +266,7 @@ func TestRequestConfigProvider_PluginRequestConfig_azureAuthEnabled(t *testing.T
 		}
 
 		p := NewRequestConfigProvider(cfg)
-		require.Subset(t, p.PluginRequestConfig(context.Background(), ""), map[string]string{"GFAZPL_AZURE_AUTH_ENABLED": "true"})
+		require.Subset(t, p.PluginRequestConfig(context.Background(), "", nil), map[string]string{"GFAZPL_AZURE_AUTH_ENABLED": "true"})
 	})
 
 	t.Run("Doesn't set the azureAuthEnabled if it is not in the config", func(t *testing.T) {
@@ -273,7 +275,7 @@ func TestRequestConfigProvider_PluginRequestConfig_azureAuthEnabled(t *testing.T
 		}
 
 		p := NewRequestConfigProvider(cfg)
-		require.NotContains(t, p.PluginRequestConfig(context.Background(), ""), "GFAZPL_AZURE_AUTH_ENABLED")
+		require.NotContains(t, p.PluginRequestConfig(context.Background(), "", nil), "GFAZPL_AZURE_AUTH_ENABLED")
 	})
 
 	t.Run("Doesn't set the azureAuthEnabled if it is false", func(t *testing.T) {
@@ -283,7 +285,7 @@ func TestRequestConfigProvider_PluginRequestConfig_azureAuthEnabled(t *testing.T
 		}
 
 		p := NewRequestConfigProvider(cfg)
-		require.NotContains(t, p.PluginRequestConfig(context.Background(), ""), "GFAZPL_AZURE_AUTH_ENABLED")
+		require.NotContains(t, p.PluginRequestConfig(context.Background(), "", nil), "GFAZPL_AZURE_AUTH_ENABLED")
 	})
 }
 
@@ -307,6 +309,7 @@ func TestRequestConfigProvider_PluginRequestConfig_azure(t *testing.T) {
 		},
 		UserIdentityFallbackCredentialsEnabled: true,
 		ForwardSettingsPlugins:                 []string{"grafana-azure-monitor-datasource", "prometheus", "grafana-azure-data-explorer-datasource", "mssql"},
+		AzureEntraPasswordCredentialsEnabled:   true,
 	}
 
 	t.Run("uses the azure settings for an Azure plugin", func(t *testing.T) {
@@ -317,7 +320,7 @@ func TestRequestConfigProvider_PluginRequestConfig_azure(t *testing.T) {
 		require.NoError(t, err)
 
 		p := NewRequestConfigProvider(pCfg)
-		require.Subset(t, p.PluginRequestConfig(context.Background(), "grafana-azure-monitor-datasource"), map[string]string{
+		require.Subset(t, p.PluginRequestConfig(context.Background(), "grafana-azure-monitor-datasource", nil), map[string]string{
 			"GFAZPL_AZURE_CLOUD": "AzureCloud", "GFAZPL_MANAGED_IDENTITY_ENABLED": "true",
 			"GFAZPL_MANAGED_IDENTITY_CLIENT_ID":                         "mock_managed_identity_client_id",
 			"GFAZPL_WORKLOAD_IDENTITY_ENABLED":                          "true",
@@ -333,6 +336,38 @@ func TestRequestConfigProvider_PluginRequestConfig_azure(t *testing.T) {
 		})
 	})
 
+	t.Run("azure settings include custom clouds when set", func(t *testing.T) {
+		cfg := setting.NewCfg()
+		cfg.Azure = azSettings
+
+		customCloudJson := `[{"name":"CustomCloud1","displayName":"Custom Cloud 1","aadAuthority":"https://login.contoso.com/","properties":null}]`
+		err := azSettings.SetCustomClouds(customCloudJson)
+		require.NoError(t, err)
+
+		// Sanity check to make sure SetCustomClouds call above also desirializes the JSON
+		require.Equal(t, len(azSettings.CustomCloudList), 1)
+
+		pCfg, err := ProvidePluginInstanceConfig(cfg, setting.ProvideProvider(cfg), featuremgmt.WithFeatures())
+		require.NoError(t, err)
+
+		p := NewRequestConfigProvider(pCfg)
+		require.Subset(t, p.PluginRequestConfig(context.Background(), "grafana-azure-monitor-datasource", nil), map[string]string{
+			"GFAZPL_AZURE_CLOUD": "AzureCloud", "GFAZPL_MANAGED_IDENTITY_ENABLED": "true",
+			"GFAZPL_MANAGED_IDENTITY_CLIENT_ID":                         "mock_managed_identity_client_id",
+			"GFAZPL_AZURE_CLOUDS_CONFIG":                                customCloudJson,
+			"GFAZPL_WORKLOAD_IDENTITY_ENABLED":                          "true",
+			"GFAZPL_WORKLOAD_IDENTITY_TENANT_ID":                        "mock_workload_identity_tenant_id",
+			"GFAZPL_WORKLOAD_IDENTITY_CLIENT_ID":                        "mock_workload_identity_client_id",
+			"GFAZPL_WORKLOAD_IDENTITY_TOKEN_FILE":                       "mock_workload_identity_token_file",
+			"GFAZPL_USER_IDENTITY_ENABLED":                              "true",
+			"GFAZPL_USER_IDENTITY_FALLBACK_SERVICE_CREDENTIALS_ENABLED": "true",
+			"GFAZPL_USER_IDENTITY_TOKEN_URL":                            "mock_user_identity_token_url",
+			"GFAZPL_USER_IDENTITY_CLIENT_ID":                            "mock_user_identity_client_id",
+			"GFAZPL_USER_IDENTITY_CLIENT_SECRET":                        "mock_user_identity_client_secret",
+			"GFAZPL_USER_IDENTITY_ASSERTION":                            "username",
+		})
+	})
+
 	t.Run("does not use the azure settings for a non-Azure plugin", func(t *testing.T) {
 		cfg := setting.NewCfg()
 		cfg.Azure = azSettings
@@ -341,7 +376,7 @@ func TestRequestConfigProvider_PluginRequestConfig_azure(t *testing.T) {
 		require.NoError(t, err)
 
 		p := NewRequestConfigProvider(pCfg)
-		m := p.PluginRequestConfig(context.Background(), "")
+		m := p.PluginRequestConfig(context.Background(), "", nil)
 		require.NotContains(t, m, "GFAZPL_AZURE_CLOUD")
 		require.NotContains(t, m, "GFAZPL_MANAGED_IDENTITY_ENABLED")
 		require.NotContains(t, m, "GFAZPL_MANAGED_IDENTITY_CLIENT_ID")
@@ -355,6 +390,7 @@ func TestRequestConfigProvider_PluginRequestConfig_azure(t *testing.T) {
 		require.NotContains(t, m, "GFAZPL_USER_IDENTITY_CLIENT_ID")
 		require.NotContains(t, m, "GFAZPL_USER_IDENTITY_CLIENT_SECRET")
 		require.NotContains(t, m, "GFAZPL_USER_IDENTITY_ASSERTION")
+		require.NotContains(t, m, "GFAZPL_AZURE_ENTRA_PASSWORD_CREDENTIALS_ENABLED")
 	})
 
 	t.Run("uses the azure settings for a non-Azure user-specified plugin", func(t *testing.T) {
@@ -366,7 +402,7 @@ func TestRequestConfigProvider_PluginRequestConfig_azure(t *testing.T) {
 		require.NoError(t, err)
 
 		p := NewRequestConfigProvider(pCfg)
-		require.Subset(t, p.PluginRequestConfig(context.Background(), "test-datasource"), map[string]string{
+		require.Subset(t, p.PluginRequestConfig(context.Background(), "test-datasource", nil), map[string]string{
 			"GFAZPL_AZURE_CLOUD": "AzureCloud", "GFAZPL_MANAGED_IDENTITY_ENABLED": "true",
 			"GFAZPL_MANAGED_IDENTITY_CLIENT_ID":                         "mock_managed_identity_client_id",
 			"GFAZPL_WORKLOAD_IDENTITY_ENABLED":                          "true",
@@ -379,6 +415,7 @@ func TestRequestConfigProvider_PluginRequestConfig_azure(t *testing.T) {
 			"GFAZPL_USER_IDENTITY_CLIENT_ID":                            "mock_user_identity_client_id",
 			"GFAZPL_USER_IDENTITY_CLIENT_SECRET":                        "mock_user_identity_client_secret",
 			"GFAZPL_USER_IDENTITY_ASSERTION":                            "username",
+			"GFAZPL_AZURE_ENTRA_PASSWORD_CREDENTIALS_ENABLED":           "true",
 		})
 	})
 }
@@ -398,7 +435,7 @@ func TestRequestConfigProvider_PluginRequestConfig_aws(t *testing.T) {
 	p := NewRequestConfigProvider(cfg)
 
 	t.Run("uses the aws settings for an AWS plugin", func(t *testing.T) {
-		require.Subset(t, p.PluginRequestConfig(context.Background(), "cloudwatch"), map[string]string{
+		require.Subset(t, p.PluginRequestConfig(context.Background(), "cloudwatch", nil), map[string]string{
 			"AWS_AUTH_AssumeRoleEnabled":     "false",
 			"AWS_AUTH_AllowedAuthProviders":  "grafana_assume_role,keys",
 			"AWS_AUTH_EXTERNAL_ID":           "mock_external_id",
@@ -408,7 +445,7 @@ func TestRequestConfigProvider_PluginRequestConfig_aws(t *testing.T) {
 	})
 
 	t.Run("does not use the aws settings for a non-aws plugin", func(t *testing.T) {
-		m := p.PluginRequestConfig(context.Background(), "")
+		m := p.PluginRequestConfig(context.Background(), "", nil)
 		require.NotContains(t, m, "AWS_AUTH_AssumeRoleEnabled")
 		require.NotContains(t, m, "AWS_AUTH_AllowedAuthProviders")
 		require.NotContains(t, m, "AWS_AUTH_EXTERNAL_ID")
@@ -420,7 +457,7 @@ func TestRequestConfigProvider_PluginRequestConfig_aws(t *testing.T) {
 		cfg.AWSForwardSettingsPlugins = append(cfg.AWSForwardSettingsPlugins, "test-datasource")
 
 		p = NewRequestConfigProvider(cfg)
-		require.Subset(t, p.PluginRequestConfig(context.Background(), "test-datasource"), map[string]string{
+		require.Subset(t, p.PluginRequestConfig(context.Background(), "test-datasource", nil), map[string]string{
 			"AWS_AUTH_AssumeRoleEnabled":     "false",
 			"AWS_AUTH_AllowedAuthProviders":  "grafana_assume_role,keys",
 			"AWS_AUTH_EXTERNAL_ID":           "mock_external_id",
@@ -429,3 +466,17 @@ func TestRequestConfigProvider_PluginRequestConfig_aws(t *testing.T) {
 		})
 	})
 }
+
+func TestRequestConfigProvider_PluginRequestConfig_appClientSecret(t *testing.T) {
+	t.Run("Uses the configured app URL", func(t *testing.T) {
+		cfg := setting.NewCfg()
+
+		pCfg, err := ProvidePluginInstanceConfig(cfg, setting.ProvideProvider(cfg), featuremgmt.WithFeatures())
+		require.NoError(t, err)
+
+		p := NewRequestConfigProvider(pCfg)
+		require.Subset(t, p.PluginRequestConfig(context.Background(), "", &auth.ExternalService{
+			ClientSecret: "mysecret",
+		}), map[string]string{backend.AppClientSecret: "mysecret"})
+	})
+}
