@@ -3,6 +3,7 @@ package provisioning
 import (
 	"context"
 	"errors"
+	"fmt"
 	"testing"
 	"time"
 
@@ -14,6 +15,7 @@ import (
 	"github.com/grafana/grafana/pkg/services/org"
 	"github.com/grafana/grafana/pkg/services/provisioning/dashboards"
 	"github.com/grafana/grafana/pkg/services/provisioning/utils"
+	"github.com/grafana/grafana/pkg/services/searchV2"
 )
 
 func TestProvisioningServiceImpl(t *testing.T) {
@@ -66,6 +68,59 @@ func TestProvisioningServiceImpl(t *testing.T) {
 		// Cancelling the root context and stopping the service
 		serviceTest.cancel()
 	})
+
+	t.Run("Should not return run error when dashboard provisioning fails because of folder", func(t *testing.T) {
+		serviceTest := setup(t)
+		provisioningErr := fmt.Errorf("%w: Test error", dashboards.ErrGetOrCreateFolder)
+		serviceTest.mock.ProvisionFunc = func(ctx context.Context) error {
+			return provisioningErr
+		}
+		err := serviceTest.service.ProvisionDashboards(context.Background())
+		assert.NotNil(t, err)
+		serviceTest.startService()
+
+		serviceTest.waitForPollChanges()
+		assert.Equal(t, 1, len(serviceTest.mock.Calls.PollChanges), "PollChanges should have been called")
+
+		// Cancelling the root context and stopping the service
+		serviceTest.cancel()
+		serviceTest.waitForStop()
+
+		assert.Equal(t, context.Canceled, serviceTest.serviceError)
+	})
+
+	t.Run("Should return run error when dashboard provisioning fails for non-allow-listed error", func(t *testing.T) {
+		serviceTest := setup(t)
+		provisioningErr := errors.New("Non-allow-listed error")
+		serviceTest.mock.ProvisionFunc = func(ctx context.Context) error {
+			return provisioningErr
+		}
+		err := serviceTest.service.ProvisionDashboards(context.Background())
+		assert.NotNil(t, err)
+		serviceTest.startService()
+
+		serviceTest.waitForPollChanges()
+		assert.Equal(t, 0, len(serviceTest.mock.Calls.PollChanges), "PollChanges should have been called")
+
+		// Cancelling the root context and stopping the service
+		serviceTest.cancel()
+		serviceTest.waitForStop()
+
+		assert.True(t, errors.Is(serviceTest.serviceError, provisioningErr))
+	})
+	t.Run("Should set dashboard provisioner when provisioning dashboards", func(t *testing.T) {
+		// The first dashboard provisioner instantiation takes place when
+		// setDashboardProvisioner() is called in setup(t).
+		serviceTest := setup(t)
+		// The second dashboard provisioner instantiation takes place when
+		// Run(ctx) is executed.
+		serviceTest.startService()
+
+		serviceTest.cancel()
+		serviceTest.waitForStop()
+
+		assert.Equal(t, 2, serviceTest.dashboardProvisionerInstantiations)
+	})
 }
 
 type serviceTestStruct struct {
@@ -79,6 +134,8 @@ type serviceTestStruct struct {
 	startService func()
 	cancel       func()
 
+	dashboardProvisionerInstantiations int
+
 	mock    *dashboards.ProvisionerMock
 	service *ProvisioningServiceImpl
 }
@@ -95,14 +152,18 @@ func setup(t *testing.T) *serviceTestStruct {
 		pollChangesChannel <- ctx
 	}
 
-	serviceTest.service = newProvisioningServiceImpl(
+	searchStub := searchV2.NewStubSearchService()
+
+	service, err := newProvisioningServiceImpl(
 		func(context.Context, string, dashboardstore.DashboardProvisioningService, org.Service, utils.DashboardStore, folder.Service) (dashboards.DashboardProvisioner, error) {
+			serviceTest.dashboardProvisionerInstantiations++
 			return serviceTest.mock, nil
 		},
 		nil,
 		nil,
+		searchStub,
 	)
-	err := serviceTest.service.setDashboardProvisioner()
+	serviceTest.service = service
 	require.NoError(t, err)
 
 	ctx, cancel := context.WithCancel(context.Background())
