@@ -16,8 +16,6 @@ import (
 )
 
 func TestIntegrationUpdateCorrelation(t *testing.T) {
-	// TODO: #82520 Possibly a flaky test
-	t.Skip()
 	if testing.Short() {
 		t.Skip("skipping integration test")
 	}
@@ -50,8 +48,6 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 			url:  fmt.Sprintf("/api/datasources/uid/%s/correlations/%s", "some-ds-uid", "some-correlation-uid"),
 			body: ``,
 		})
-		require.Equal(t, http.StatusUnauthorized, res.StatusCode)
-
 		responseBody, err := io.ReadAll(res.Body)
 		require.NoError(t, err)
 
@@ -60,6 +56,7 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 		require.NoError(t, err)
 
 		require.Equal(t, "Unauthorized", response.Message)
+		require.Equal(t, http.StatusUnauthorized, res.StatusCode)
 
 		require.NoError(t, res.Body.Close())
 	})
@@ -70,8 +67,6 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 			body: `{}`,
 			user: editorUser,
 		})
-		require.Equal(t, http.StatusForbidden, res.StatusCode)
-
 		responseBody, err := io.ReadAll(res.Body)
 		require.NoError(t, err)
 
@@ -80,6 +75,7 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 		require.NoError(t, err)
 
 		require.Contains(t, response.Message, "Permissions needed: datasources:write")
+		require.Equal(t, http.StatusForbidden, res.StatusCode)
 
 		require.NoError(t, res.Body.Close())
 	})
@@ -92,8 +88,6 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 			}`,
 			user: adminUser,
 		})
-		require.Equal(t, http.StatusNotFound, res.StatusCode)
-
 		responseBody, err := io.ReadAll(res.Body)
 		require.NoError(t, err)
 
@@ -102,6 +96,7 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 		require.NoError(t, err)
 
 		require.Equal(t, "Data source not found", response.Message)
+		require.Equal(t, http.StatusNotFound, res.StatusCode)
 
 		require.NoError(t, res.Body.Close())
 	})
@@ -114,8 +109,6 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 				"label": ""
 			}`,
 		})
-		require.Equal(t, http.StatusNotFound, res.StatusCode)
-
 		responseBody, err := io.ReadAll(res.Body)
 		require.NoError(t, err)
 
@@ -124,6 +117,7 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 		require.NoError(t, err)
 
 		require.Equal(t, "Correlation not found", response.Message)
+		require.Equal(t, http.StatusNotFound, res.StatusCode)
 
 		require.NoError(t, res.Body.Close())
 	})
@@ -134,6 +128,7 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 			TargetUID:   &writableDs,
 			OrgId:       writableDsOrgId,
 			Provisioned: true,
+			Type:        correlations.CorrelationType("query"),
 		})
 
 		res := ctx.Patch(PatchParams{
@@ -143,8 +138,6 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 				"label": "some-label"
 			}`,
 		})
-		require.Equal(t, http.StatusForbidden, res.StatusCode)
-
 		responseBody, err := io.ReadAll(res.Body)
 		require.NoError(t, err)
 
@@ -153,6 +146,7 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 		require.NoError(t, err)
 
 		require.Equal(t, "Correlation can only be edited via provisioning", response.Message)
+		require.Equal(t, http.StatusForbidden, res.StatusCode)
 
 		require.NoError(t, res.Body.Close())
 	})
@@ -162,6 +156,7 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 			SourceUID: writableDs,
 			TargetUID: &writableDs,
 			OrgId:     writableDsOrgId,
+			Type:      correlations.CorrelationType("query"),
 		})
 
 		// no params
@@ -170,8 +165,6 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 			user: adminUser,
 			body: `{}`,
 		})
-		require.Equal(t, http.StatusBadRequest, res.StatusCode)
-
 		responseBody, err := io.ReadAll(res.Body)
 		require.NoError(t, err)
 
@@ -180,6 +173,8 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 		require.NoError(t, err)
 
 		require.Equal(t, "At least one of label, description or config is required", response.Message)
+		require.Equal(t, http.StatusBadRequest, res.StatusCode)
+
 		require.NoError(t, res.Body.Close())
 
 		// empty body
@@ -188,8 +183,6 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 			user: adminUser,
 			body: ``,
 		})
-		require.Equal(t, http.StatusBadRequest, res.StatusCode)
-
 		responseBody, err = io.ReadAll(res.Body)
 		require.NoError(t, err)
 
@@ -197,6 +190,8 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 		require.NoError(t, err)
 
 		require.Equal(t, "At least one of label, description or config is required", response.Message)
+		require.Equal(t, http.StatusBadRequest, res.StatusCode)
+
 		require.NoError(t, res.Body.Close())
 
 		// all set to null
@@ -209,8 +204,6 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 						"config": null
 					}`,
 		})
-		require.Equal(t, http.StatusBadRequest, res.StatusCode)
-
 		responseBody, err = io.ReadAll(res.Body)
 		require.NoError(t, err)
 
@@ -218,6 +211,8 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 		require.NoError(t, err)
 
 		require.Equal(t, "At least one of label, description or config is required", response.Message)
+		require.Equal(t, http.StatusBadRequest, res.StatusCode)
+
 		require.NoError(t, res.Body.Close())
 	})
 
@@ -227,6 +222,7 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 			TargetUID: &writableDs,
 			OrgId:     writableDsOrgId,
 			Label:     "a label",
+			Type:      correlations.CorrelationType("query"),
 		})
 
 		res := ctx.Patch(PatchParams{
@@ -236,8 +232,6 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 				"label": "updated label"
 			}`,
 		})
-		require.Equal(t, http.StatusOK, res.StatusCode)
-
 		responseBody, err := io.ReadAll(res.Body)
 		require.NoError(t, err)
 
@@ -246,6 +240,7 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 		require.NoError(t, err)
 
 		require.Equal(t, "Correlation updated", response.Message)
+		require.Equal(t, http.StatusOK, res.StatusCode)
 		require.Equal(t, "updated label", response.Result.Label)
 		require.NoError(t, res.Body.Close())
 	})
@@ -257,9 +252,9 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 			OrgId:       writableDsOrgId,
 			Label:       "0",
 			Description: "0",
+			Type:        correlations.CorrelationType("query"),
 			Config: correlations.CorrelationConfig{
 				Field:  "fieldName",
-				Type:   "query",
 				Target: map[string]any{"expr": "foo"},
 			},
 		})
@@ -271,15 +266,14 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 			body: `{
 				"label": "1",
 				"description": "1",
+				"type": "query",
 				"config": {
 					"field": "field",
-					"type": "query",
 					"target": { "expr": "bar" },
 					"transformations": [ {"type": "logfmt"} ]
 				}
 			}`,
 		})
-		require.Equal(t, http.StatusOK, res.StatusCode)
 
 		responseBody, err := io.ReadAll(res.Body)
 		require.NoError(t, err)
@@ -289,160 +283,12 @@ func TestIntegrationUpdateCorrelation(t *testing.T) {
 		require.NoError(t, err)
 
 		require.Equal(t, "Correlation updated", response.Message)
+		require.Equal(t, http.StatusOK, res.StatusCode)
 		require.Equal(t, "1", response.Result.Label)
 		require.Equal(t, "1", response.Result.Description)
 		require.Equal(t, "field", response.Result.Config.Field)
 		require.Equal(t, map[string]any{"expr": "bar"}, response.Result.Config.Target)
 		require.Equal(t, correlations.Transformation{Type: "logfmt"}, response.Result.Config.Transformations[0])
 		require.NoError(t, res.Body.Close())
-
-		// partially updating only label
-		res = ctx.Patch(PatchParams{
-			url:  fmt.Sprintf("/api/datasources/uid/%s/correlations/%s", correlation.SourceUID, correlation.UID),
-			user: adminUser,
-			body: `{
-				"label": "2"
-			}`,
-		})
-		require.Equal(t, http.StatusOK, res.StatusCode)
-
-		responseBody, err = io.ReadAll(res.Body)
-		require.NoError(t, err)
-
-		err = json.Unmarshal(responseBody, &response)
-		require.NoError(t, err)
-
-		require.Equal(t, "Correlation updated", response.Message)
-		require.Equal(t, "2", response.Result.Label)
-		require.Equal(t, "1", response.Result.Description)
-		require.Equal(t, "field", response.Result.Config.Field)
-		require.Equal(t, map[string]any{"expr": "bar"}, response.Result.Config.Target)
-		require.NoError(t, res.Body.Close())
-
-		// partially updating only description
-		res = ctx.Patch(PatchParams{
-			url:  fmt.Sprintf("/api/datasources/uid/%s/correlations/%s", correlation.SourceUID, correlation.UID),
-			user: adminUser,
-			body: `{
-				"description": "2"
-			}`,
-		})
-		require.Equal(t, http.StatusOK, res.StatusCode)
-
-		responseBody, err = io.ReadAll(res.Body)
-		require.NoError(t, err)
-
-		err = json.Unmarshal(responseBody, &response)
-		require.NoError(t, err)
-
-		require.Equal(t, "Correlation updated", response.Message)
-		require.Equal(t, "2", response.Result.Label)
-		require.Equal(t, "2", response.Result.Description)
-		require.Equal(t, "field", response.Result.Config.Field)
-		require.Equal(t, map[string]any{"expr": "bar"}, response.Result.Config.Target)
-		require.NoError(t, res.Body.Close())
-
-		// partially updating whole config
-		res = ctx.Patch(PatchParams{
-			url:  fmt.Sprintf("/api/datasources/uid/%s/correlations/%s", correlation.SourceUID, correlation.UID),
-			user: adminUser,
-			body: `{
-				"config": {
-					"field": "name",
-					"type": "query",
-					"target": { "expr": "baz" }
-				}
-			}`,
-		})
-		require.Equal(t, http.StatusOK, res.StatusCode)
-
-		responseBody, err = io.ReadAll(res.Body)
-		require.NoError(t, err)
-
-		err = json.Unmarshal(responseBody, &response)
-		require.NoError(t, err)
-
-		require.Equal(t, "Correlation updated", response.Message)
-		require.Equal(t, "2", response.Result.Label)
-		require.Equal(t, "2", response.Result.Description)
-		require.Equal(t, "name", response.Result.Config.Field)
-		require.Equal(t, map[string]any{"expr": "baz"}, response.Result.Config.Target)
-		require.NoError(t, res.Body.Close())
-
-		// partially updating only config field
-		res = ctx.Patch(PatchParams{
-			url:  fmt.Sprintf("/api/datasources/uid/%s/correlations/%s", correlation.SourceUID, correlation.UID),
-			user: adminUser,
-			body: `{
-				"config": {
-					"field": "newName"
-				}
-			}`,
-		})
-		require.Equal(t, http.StatusOK, res.StatusCode)
-
-		responseBody, err = io.ReadAll(res.Body)
-		require.NoError(t, err)
-
-		err = json.Unmarshal(responseBody, &response)
-		require.NoError(t, err)
-
-		require.Equal(t, "Correlation updated", response.Message)
-		require.Equal(t, "2", response.Result.Label)
-		require.Equal(t, "2", response.Result.Description)
-		require.Equal(t, "newName", response.Result.Config.Field)
-		require.Equal(t, map[string]any{"expr": "baz"}, response.Result.Config.Target)
-		require.NoError(t, res.Body.Close())
-
-		// partially updating only config target
-		res = ctx.Patch(PatchParams{
-			url:  fmt.Sprintf("/api/datasources/uid/%s/correlations/%s", correlation.SourceUID, correlation.UID),
-			user: adminUser,
-			body: `{
-				"config": {
-					"target": { "expr": "foo" }
-				}
-			}`,
-		})
-		require.Equal(t, http.StatusOK, res.StatusCode)
-
-		responseBody, err = io.ReadAll(res.Body)
-		require.NoError(t, err)
-
-		err = json.Unmarshal(responseBody, &response)
-		require.NoError(t, err)
-
-		require.Equal(t, "Correlation updated", response.Message)
-		require.Equal(t, "2", response.Result.Label)
-		require.Equal(t, "2", response.Result.Description)
-		require.Equal(t, "newName", response.Result.Config.Field)
-		require.Equal(t, map[string]any{"expr": "foo"}, response.Result.Config.Target)
-		require.NoError(t, res.Body.Close())
-
-		// setting label, description and config field to empty strings (testing whether empty strings are handled correctly)
-		res = ctx.Patch(PatchParams{
-			url:  fmt.Sprintf("/api/datasources/uid/%s/correlations/%s", correlation.SourceUID, correlation.UID),
-			user: adminUser,
-			body: `{
-				"label": "",
-				"description": "",
-				"config": {
-					"field": ""
-				}
-			}`,
-		})
-		require.Equal(t, http.StatusOK, res.StatusCode)
-
-		responseBody, err = io.ReadAll(res.Body)
-		require.NoError(t, err)
-
-		err = json.Unmarshal(responseBody, &response)
-		require.NoError(t, err)
-
-		require.Equal(t, "Correlation updated", response.Message)
-		require.Equal(t, "", response.Result.Label)
-		require.Equal(t, "", response.Result.Description)
-		require.Equal(t, "", response.Result.Config.Field)
-		require.NoError(t, res.Body.Close())
 	})
 }
