@@ -4,6 +4,7 @@ import (
 	"context"
 	"encoding/base64"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"math"
 	"math/rand"
@@ -198,6 +199,12 @@ Timestamps will line up evenly on timeStepSeconds (For example, 60 seconds means
 		Name: "Trace",
 	})
 
+	s.registerScenario(&Scenario{
+		ID:      kinds.TestDataQueryTypeErrorWithSource,
+		Name:    "Error with source",
+		handler: s.handleErrorWithSourceScenario,
+	})
+
 	s.queryMux.HandleFunc("", s.handleFallbackScenario)
 }
 
@@ -215,7 +222,7 @@ func instrumentScenarioHandler(logger log.Logger, scenario kinds.TestDataQueryTy
 		defer span.End()
 
 		ctxLogger := logger.FromContext(ctx)
-		ctxLogger.Debug("queryData", "scenario", scenario)
+		ctxLogger.Debug(string(backend.EndpointQueryData), "scenario", scenario)
 
 		return fn(ctx, req)
 	})
@@ -663,6 +670,29 @@ func (s *Service) handleLogsScenario(ctx context.Context, req *backend.QueryData
 	return resp, nil
 }
 
+func (s *Service) handleErrorWithSourceScenario(ctx context.Context, req *backend.QueryDataRequest) (*backend.QueryDataResponse, error) {
+	anErr := errors.New("error")
+	resp := backend.NewQueryDataResponse()
+
+	for _, q := range req.Queries {
+		model, err := GetJSONModel(q.JSON)
+		if err != nil {
+			continue
+		}
+
+		respD := resp.Responses[q.RefID]
+		respD.Error = anErr
+
+		if model.ErrorSource == kinds.ErrorSourceDownstream {
+			respD.Error = backend.DownstreamError(respD.Error)
+		}
+
+		resp.Responses[q.RefID] = respD
+	}
+
+	return resp, nil
+}
+
 func RandomWalk(query backend.DataQuery, model kinds.TestDataQuery, index int) *data.Frame {
 	rand := rand.New(rand.NewSource(time.Now().UnixNano() + int64(index)))
 	timeWalkerMs := query.TimeRange.From.UnixNano() / int64(time.Millisecond)
