@@ -94,6 +94,12 @@ func (session *Session) AllCols() *Session {
 	return session
 }
 
+// AllCols ask all columns
+func (session *Session) Omit(columns ...string) *Session {
+	session.statement.Omit(columns...)
+	return session
+}
+
 // MustCols specify some columns must use even if they are empty
 func (session *Session) MustCols(columns ...string) *Session {
 	session.statement.MustCols(columns...)
@@ -123,3 +129,10 @@ func (session *Session) Nullable(columns ...string) *Session {
 	session.statement.Nullable(columns...)
 	return session
 }
+
+// NoAutoTime means do not automatically give created field and updated field
+// the current time on the current session temporarily
+func (session *Session) NoAutoTime() *Session {
+	session.statement.UseAutoTime = false
+	return session
+}
