@@ -1,5 +1,7 @@
 const fs = require('fs');
 
+const printAffectedPluginsSection = require('./levitate-show-affected-plugins');
+
 const data = JSON.parse(fs.readFileSync('data.json', 'utf8'));
 
 function stripAnsi(str) {
@@ -28,4 +30,8 @@ if (data.changes.length > 0) {
   markdown += printSection('Changes', data.changes);
 }
 
+if (data.removals.length > 0 || data.changes.length > 0) {
+  markdown += printAffectedPluginsSection(data);
+}
+
 console.log(markdown);
